#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from tqdm import tqdm
import time
import zipfile
import tempfile
import shutil
import os

from enum import Enum

from abxr.api_service import ApiService
from abxr.constants import SYSTEM_FILES_TO_EXCLUDE, SYSTEM_DIRS_TO_EXCLUDE
from abxr.multipart import MultipartFileS3
from abxr.formats import DataOutputFormats
from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"
    DETAILS = "details"
    VERSION_LIST = "versions"
    RELEASE_CHANNELS_LIST = "release_channels"
    RELEASE_CHANNEL_DETAILS = "release_channel_details"
    RELEASE_CHANNEL_SET_VERSION = "release_channel_set_version"
    UPLOAD = "upload"
    SHARE = "share"
    REVOKE_SHARE = "revoke"

class AppsService(ApiService):
    MAX_PARTS_PER_REQUEST = 4

    def _initiate_upload(self, app_id, file_name, app_build_type="standalone", release_channel_id=None, new_release_channel_title=None):
        url = self._url('apps', app_id, 'versions')
        data = {'filename': file_name}

        if app_build_type:
            data['appBuildType'] = app_build_type

        if release_channel_id:
            data['releaseChannelId'] = release_channel_id

        if new_release_channel_title:
            data['newReleaseChannelTitle'] = new_release_channel_title

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def _presigned_url(self, app_id, version_id, upload_id, key, part_numbers):
        url = self._url('apps', app_id, 'versions', version_id, 'pre-sign')
        data = {'key': key,
                'uploadId': upload_id,
                'partNumbers': part_numbers
                }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def _complete_upload(self, app_id, version_id, upload_id, key, parts, version_name, release_notes):
        url = self._url('apps', app_id, 'versions', version_id, 'complete')
        data = {'key': key,
                'uploadId': upload_id,
                'parts': parts,
                'versionName': version_name,
                'releaseNotes': release_notes
                }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def upload_file(self, app_id, file_path, version_number, release_notes, silent, wait, max_wait_time_sec=60, app_build_type="standalone", release_channel_id=None, new_release_channel_title=None):
        file = MultipartFileS3(file_path)

        response = self._initiate_upload(app_id, file.file_name, app_build_type, release_channel_id, new_release_channel_title)

        upload_id = response['uploadId']
        key = response['key']
        version_id = response['versionId']
        app_bundle_id = response.get('appBundleId')  # Capture bundle ID if created

        part_numbers = list(range(1, file.get_part_numbers() + 1))

        uploaded_parts = []

        with tqdm(total=file.get_size(), unit='B', unit_scale=True, desc=f'Uploading {file.file_name}', disable=silent) as pbar:
            for i in range(0, len(part_numbers), self.MAX_PARTS_PER_REQUEST):
                part_numbers_slice = part_numbers[i:i + self.MAX_PARTS_PER_REQUEST]

                presigned_url_response = self._presigned_url(app_id, version_id, upload_id, key, part_numbers_slice)

                for item in presigned_url_response:
                    part_number = item['partNumber']
                    presigned_url = item['presignedUrl']

                    part = file.get_part(part_number)
                    response = self.client.put(presigned_url, data=part)
                    response.raise_for_status()

                    uploaded_parts += [{'partNumber': part_number, 'eTag': response.headers['ETag']}]
                    pbar.update(len(part))

            complete_response = self._complete_upload(app_id, version_id, upload_id, key, uploaded_parts, version_number, release_notes)

            total_time_sec = 0
            wait_indefinitely = max_wait_time_sec <= 0

            if wait_indefinitely:
                max_wait_time_sec = 1

            status = None

            if wait:
                while status != 'available' and total_time_sec < max_wait_time_sec:
                    versions = self.get_all_versions_for_app(app_id)
                    version = next((v for v in versions if v['id'] == version_id), None)
                    if version:
                        status = version['status']
                        if status == 'available':
                            break
                        elif status == 'error':
                            raise Exception(f"Upload failed server processing for version {version_id} of app {app_id}.")
                    else:
                        raise Exception(f"Version {version_id} not found for uploaded app {app_id}.")

                    pbar.set_description(f'Been waiting for upload to complete for {total_time_sec} seconds')
                    time.sleep(1)
                    total_time_sec += 1

                    if wait_indefinitely:
                        max_wait_time_sec += 1

            # Include appBundleId in response if it was created
            if app_bundle_id:
                complete_response['appBundleId'] = app_bundle_id

            return complete_response

    def get_all_apps(self):
        url = self._url('apps') + '?per_page=20'
        return self._get_all_pages(url)

    def get_app_detail(self, app_id):
        url = self._url('apps', app_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def get_all_versions_for_app(self, app_id):
        url = self._url('apps', app_id, 'versions')
        return self._get_all_pages(url)

    def get_versions_by_sha256(self, app_id, sha256_hashes):
        """Query app versions by SHA-256 hashes, return only available versions"""
        if not sha256_hashes:
            return []

        query_params = '&'.join([f'sha256[]={hash}' for hash in sha256_hashes])
        url = self._url('apps', app_id, 'versions') + '?' + query_params

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        json_data = response.json()
        data = json_data.get('data', [])

        return [v for v in data if v.get('status') == 'available']

    def get_files_by_sha512(self, app_id, sha512_hashes):
        """Query app files by SHA-512 hashes"""
        if not sha512_hashes:
            return []

        # Build query string with sha512[] array parameters
        query_params = '&'.join([f'sha512[]={hash}' for hash in sha512_hashes])
        url = self._url('apps', app_id, 'files') + '?' + query_params

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        json_data = response.json()
        return json_data.get('data', [])

    def get_all_release_channels_for_app(self, app_id):
        url = self._url('apps', app_id, 'release-channels')
        return self._get_all_pages(url)

    def get_release_channel_detail(self, app_id, release_channel_id):
        url = self._url('apps', app_id, 'release-channels', release_channel_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def set_version_for_release_channel(self, app_id, release_channel_id, version_id=None, bundle_id=None):
        url = self._url('apps', app_id, 'release-channels', release_channel_id)

        data = {'bundleId': bundle_id} if bundle_id else {'versionId': version_id}

        response = self.client.put(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def share_app(self, app_id, release_channel_id, organization_slug):
        url = self._url('apps', app_id, 'release-channels', release_channel_id, 'share')
        data = {'organizationSlug': organization_slug}

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def revoke_shared_app(self, app_id, release_channel_id, organization_slug):
        url = self._url('apps', app_id, 'release-channels', release_channel_id, 'share')
        data = {'organizationSlug': organization_slug}

        response = self.client.delete(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

def _handle_zip_upload(args, apps_service):
    """Handle ZIP file upload by extracting and converting to bundle or regular upload

    Args:
        args: Command-line arguments
        apps_service: AppsService instance

    Returns:
        Upload response
    """
    zip_path = args.filename

    if not args.silent:
        print(f"Extracting ZIP file...")

    # Create temporary directory
    temp_dir = tempfile.mkdtemp(prefix='abxr_zip_')

    try:
        # Extract ZIP
        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall(temp_dir)

        # Find all APK files at root (case-insensitive, no subdirectories)
        apk_files = []
        for item in os.listdir(temp_dir):
            item_path = os.path.join(temp_dir, item)
            if os.path.isfile(item_path) and item.lower().endswith('.apk'):
                apk_files.append(item_path)

        # Validate APK count
        if len(apk_files) == 0:
            raise ValueError("No APK file found at root of ZIP")
        elif len(apk_files) > 1:
            raise ValueError(f"Multiple APK files found at root of ZIP ({len(apk_files)} APKs found)")

        apk_path = apk_files[0]

        # Find all other files (excluding system files)
        other_files = []
        for root, dirs, files in os.walk(temp_dir):
            # Skip system directories
            if any(sys_dir in root for sys_dir in SYSTEM_DIRS_TO_EXCLUDE):
                continue
            for file in files:
                file_path = os.path.join(root, file)
                # Skip if it's the APK or a system file
                if file_path != apk_path and file not in SYSTEM_FILES_TO_EXCLUDE:
                    other_files.append(file_path)

        # If only APK, use normal upload flow
        if not other_files:
            if not args.silent:
                print(f"ZIP contains only APK, using standard upload...")
            release_channel_id = getattr(args, 'release_channel_id', None)
            new_release_channel_title = getattr(args, 'new_release_channel_title', None)
            result = apps_service.upload_file(
                args.app_id,
                apk_path,
                args.version_number,
                args.notes,
                args.silent,
                args.wait,
                args.wait_time,
                release_channel_id=release_channel_id,
                new_release_channel_title=new_release_channel_title
            )
            return result

        # ZIP contains APK + other files - create bundle
        if not args.silent:
            print(f"ZIP contains APK and {len(other_files)} additional file(s), creating app bundle...")

        # Get app details to retrieve package name
        app_detail = apps_service.get_app_detail(args.app_id)
        package_name = app_detail.get('packageName')

        if not package_name:
            raise ValueError("Cannot create bundle from ZIP: app does not have a package name set. Upload an APK first to set the package name.")

        # Restructure: move OBB files to Android/obb/<package-name>/
        obb_dir = os.path.join(temp_dir, 'Android', 'obb', package_name)
        for file_path in other_files:
            if file_path.lower().endswith('.obb'):
                # Create OBB directory if needed
                os.makedirs(obb_dir, exist_ok=True)
                # Move OBB file
                file_name = os.path.basename(file_path)
                new_path = os.path.join(obb_dir, file_name)
                shutil.move(file_path, new_path)
                if not args.silent:
                    print(f"Moved {file_name} to Android/obb/{package_name}/")

        # Import AppBundlesService
        from abxr.app_bundles import AppBundlesService

        # Create bundle service
        bundle_service = AppBundlesService(args.url, args.token)

        # Upload as bundle
        release_channel_id = getattr(args, 'release_channel_id', None)
        new_release_channel_title = getattr(args, 'new_release_channel_title', None)
        result = bundle_service.upload_app_bundle(
            args.app_id,
            temp_dir,
            args.version_number,
            args.notes,
            args.silent,
            apk_path=apk_path,
            device_path=None,
            release_channel_id=release_channel_id,
            new_release_channel_title=new_release_channel_title
        )

        return result

    finally:
        # Clean up temp directory
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)

class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = AppsService(self.args.url, self.args.token)

    def run(self):
        if self.args.apps_command == Commands.LIST.value:
            apps_list = self.service.get_all_apps()
            print_formatted(self.args.format, apps_list)

        elif self.args.apps_command == Commands.DETAILS.value:
            app_detail = self.service.get_app_detail(self.args.app_id)
            print_formatted(self.args.format, app_detail)

        elif self.args.apps_command == Commands.RELEASE_CHANNELS_LIST.value:
            release_channels = self.service.get_all_release_channels_for_app(self.args.app_id)
            print_formatted(self.args.format, release_channels)

        elif self.args.apps_command == Commands.RELEASE_CHANNEL_DETAILS.value:
            release_channel_detail = self.service.get_release_channel_detail(self.args.app_id, self.args.release_channel_id)
            print_formatted(self.args.format, release_channel_detail)

        elif self.args.apps_command == Commands.VERSION_LIST.value:
            versions = self.service.get_all_versions_for_app(self.args.app_id)
            print_formatted(self.args.format, versions)

        elif self.args.apps_command == Commands.RELEASE_CHANNEL_SET_VERSION.value:
            self.service.set_version_for_release_channel(self.args.app_id, self.args.release_channel_id, version_id=self.args.version_id, bundle_id=self.args.bundle_id)

        elif self.args.apps_command == Commands.UPLOAD.value:
            # Check if file is a ZIP - if so, extract and convert to bundle or regular upload
            if self.args.filename.lower().endswith('.zip'):
                app_version = _handle_zip_upload(self.args, self.service)
            else:
                release_channel_id = getattr(self.args, 'release_channel_id', None)
                new_release_channel_title = getattr(self.args, 'new_release_channel_title', None)
                app_version = self.service.upload_file(
                    self.args.app_id,
                    self.args.filename,
                    self.args.version_number,
                    self.args.notes,
                    self.args.silent,
                    self.args.wait,
                    self.args.wait_time,
                    release_channel_id=release_channel_id,
                    new_release_channel_title=new_release_channel_title
                )
            print_formatted(self.args.format, app_version)

        elif self.args.apps_command == Commands.SHARE.value:
            self.service.share_app(self.args.app_id, self.args.release_channel_id, self.args.organization_slug)

        elif self.args.apps_command == Commands.REVOKE_SHARE.value:
            self.service.revoke_shared_app(self.args.app_id, self.args.release_channel_id, self.args.organization_slug)



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from tqdm import tqdm

from enum import Enum

from abxr.api_service import ApiService
from abxr.multipart import MultipartFileS3
from abxr.formats import DataOutputFormats
from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"
    DETAILS = "details"
    UPDATE = "update"
    UPLOAD = "upload"
    ATTACH_TAGS = "attach_tags"
    DETACH_TAGS = "detach_tags"

class VideosService(ApiService):
    MAX_PARTS_PER_REQUEST = 4

    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def _initiate_upload(self, file_name, video_type, video_mapping, video_display, video_packing, audio_encoding):
        url = self._url('videos')

        data = {
            'filename': file_name,
            'videoType': video_type,
            'videoMapping': video_mapping,
            'videoDisplay': video_display,
            'videoPacking': video_packing,
            'audioEncoding': audio_encoding
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def _presigned_url(self, video_id, upload_id, key, part_numbers):
        url = self._url('videos', video_id, 'pre-sign')
        data = {
            'key': key,
            'uploadId': upload_id,
            'partNumbers': part_numbers
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def _complete_upload(self, video_id, upload_id, key, parts):
        url = self._url('videos', video_id, 'complete')
        data = {'key': key,
                'uploadId': upload_id,
                'parts': parts
                }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def upload_video(self, file_path, video_type, video_mapping, video_display, video_packing, audio_encoding, silent):
        file = MultipartFileS3(file_path)

        response = self._initiate_upload(file.file_name, video_type, video_mapping, video_display, video_packing, audio_encoding)

        upload_id = response['uploadId']
        key = response['key']
        video_id = response['videoId']

        part_numbers = list(range(1, file.get_part_numbers() + 1))

        uploaded_parts = []

        with tqdm(total=file.get_size(), unit='B', unit_scale=True, desc=f'Uploading {file.file_name}', disable=silent) as pbar:
            for i in range(0, len(part_numbers), self.MAX_PARTS_PER_REQUEST):
                part_numbers_slice = part_numbers[i:i + self.MAX_PARTS_PER_REQUEST]

                presigned_url_response = self._presigned_url(video_id, upload_id, key, part_numbers_slice)

                for item in presigned_url_response:
                    part_number = item['partNumber']
                    presigned_url = item['presignedUrl']

                    part = file.get_part(part_number)
                    response = self.client.put(presigned_url, data=part)
                    response.raise_for_status()

                    uploaded_parts += [{'partNumber': part_number, 'eTag': response.headers['ETag']}]
                    pbar.update(len(part))

            complete_response = self._complete_upload(video_id, upload_id, key, uploaded_parts)
            return complete_response

    def get_all_videos(self):
        url = self._url('videos') + '?per_page=20'
        return self._get_all_pages(url)

    def get_video_detail(self, video_id):
        url = self._url('videos', video_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def update_video(self, video_id, name, description, video_type, video_mapping, video_display, video_package, audio_encoding, tags):
        url = self._url('videos', video_id)

        payload = {
            'name': name,
            'description': description,
            'videoType': video_type,
            'videoMapping': video_mapping,
            'videoDisplay': video_display,
            'videoPacking': video_package,
            'audioEncoding': audio_encoding,
            'tags': tags
        }

        response = self.client.put(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def add_tags_to_video(self, video_id, tags):
        url = self._url('videos', video_id, 'tags', 'attach')

        payload = {
            'tags': tags
        }

        response = self.client.patch(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def remove_tags_to_video(self, video_id, tags):
        url = self._url('videos', video_id, 'tags', 'detach')

        payload = {
            'tags': tags
        }

        response = self.client.patch(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()


class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = VideosService(self.args.url, self.args.token)

    def run(self):
        if self.args.videos_command == Commands.LIST.value:
            videos_list = self.service.get_all_videos()
            print_formatted(self.args.format, videos_list)

        elif self.args.videos_command == Commands.DETAILS.value:
            video_detail = self.service.get_video_detail(self.args.video_id)
            print_formatted(self.args.format, video_detail)

        elif self.args.videos_commands == Commands.UPDATE.value:
            video_detail = self.service.update_video(self.args.video_id, self.args.name, self.args.description, self.args.video_type, self.args.video_mapping, self.args.video_display, self.args.video_packing, self.args.audio_encoding, self.args.tags)
            print_formatted(self.args.format, video_detail)

        elif self.args.videos_command == Commands.UPLOAD.value:
            video_version = self.service.upload_video(self.args.filename, self.args.video_type, self.args.video_mapping, self.args.video_display, self.args.video_packing, self.args.audio_encoding, self.args.silent)
            print_formatted(self.args.format, video_version)

        elif self.args.videos_command == Commands.ATTACH_TAGS.value:
            self.service.add_tags_to_video(self.args.video_id, self.args.tags)

        elif self.args.videos_command == Commands.DETACH_TAGS.value:
            self.service.remove_tags_to_video(self.args.video_id, self.args.tags)



from pathlib import Path

import pytest

from abxr.app_bundles import AppBundlesService
from tests.conftest import make_response


def _service(mocker):
    svc = AppBundlesService("https://api.xrdm.app", "tok")
    svc.client = mocker.MagicMock()
    return svc


class TestDevicePathComputation:
    def test_root_file_default(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "config.json"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard"

    def test_subdir_file_default(self, mocker, tmp_path):
        svc = _service(mocker)
        (tmp_path / "data" / "cache").mkdir(parents=True)
        f = tmp_path / "data" / "cache" / "temp.dat"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard/data/cache"

    def test_root_file_with_base_path(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "config.json"
        f.touch()
        assert svc._compute_device_path(f, tmp_path, "myapp/config") == "/sdcard/myapp/config"

    def test_subdir_file_with_base_path(self, mocker, tmp_path):
        svc = _service(mocker)
        (tmp_path / "data").mkdir()
        f = tmp_path / "data" / "user.json"
        f.touch()
        assert svc._compute_device_path(f, tmp_path, "myapp/config") == "/sdcard/myapp/config/data"

    def test_main_obb_routed_to_canonical_path(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "main.123.com.example.testapp.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard/Android/obb/com.example.testapp"

    def test_patch_obb_routed_to_canonical_path(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "patch.123.com.example.testapp.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard/Android/obb/com.example.testapp"

    def test_obb_in_subdir_ignores_folder_layout(self, mocker, tmp_path):
        svc = _service(mocker)
        (tmp_path / "extras").mkdir()
        f = tmp_path / "extras" / "main.1.com.example.app.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard/Android/obb/com.example.app"

    def test_obb_bypasses_base_path(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "main.7.com.foo.bar.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path, "custom/path") == "/sdcard/Android/obb/com.foo.bar"

    def test_misnamed_obb_falls_back_with_warning(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        f = tmp_path / "weird.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard"
        assert "does not match the expected naming convention" in capsys.readouterr().out

    def test_obb_with_non_numeric_version_falls_back(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        f = tmp_path / "main.notanumber.com.foo.bar.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard"
        assert "does not match the expected naming convention" in capsys.readouterr().out

    def test_obb_with_single_segment_package_falls_back(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        f = tmp_path / "main.1.foo.obb"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard"
        assert "does not match the expected naming convention" in capsys.readouterr().out

    def test_uppercase_extension_treated_as_non_obb(self, mocker, tmp_path, capsys):
        # Android's OBB loader is case-sensitive; only lowercase .obb counts.
        # A file with uppercase .OBB should get the default path with no warning.
        svc = _service(mocker)
        f = tmp_path / "main.1.com.example.app.OBB"
        f.touch()
        assert svc._compute_device_path(f, tmp_path) == "/sdcard"
        assert capsys.readouterr().out == ""


class TestObbPackageConsistency:
    def test_no_warning_for_single_package_with_main_and_patch(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        files = [
            tmp_path / "main.1.com.example.app.obb",
            tmp_path / "patch.1.com.example.app.obb",
        ]
        for f in files:
            f.touch()
        svc._warn_on_mixed_obb_packages(files)
        out = capsys.readouterr().out
        assert "multiple packages" not in out

    def test_warns_when_obbs_declare_different_packages(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        files = [
            tmp_path / "main.1.com.foo.app.obb",
            tmp_path / "main.1.com.bar.app.obb",
        ]
        for f in files:
            f.touch()
        svc._warn_on_mixed_obb_packages(files)
        out = capsys.readouterr().out
        assert "multiple packages" in out
        assert "com.foo.app" in out
        assert "com.bar.app" in out
        assert "main.1.com.foo.app.obb" in out
        assert "main.1.com.bar.app.obb" in out

    def test_misnamed_obb_does_not_count_as_separate_package(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        files = [
            tmp_path / "main.1.com.example.app.obb",
            tmp_path / "weird.obb",
        ]
        for f in files:
            f.touch()
        svc._warn_on_mixed_obb_packages(files)
        out = capsys.readouterr().out
        assert "multiple packages" not in out

    def test_no_obbs_at_all(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        files = [tmp_path / "config.json", tmp_path / "data.bin"]
        for f in files:
            f.touch()
        svc._warn_on_mixed_obb_packages(files)
        assert capsys.readouterr().out == ""

    def test_uppercase_obb_extension_does_not_count(self, mocker, tmp_path, capsys):
        svc = _service(mocker)
        files = [
            tmp_path / "main.1.com.example.app.obb",
            tmp_path / "main.1.com.other.app.OBB",
        ]
        for f in files:
            f.touch()
        svc._warn_on_mixed_obb_packages(files)
        assert capsys.readouterr().out == ""


class TestValidateBundleFilesMatch:
    def test_extracts_filename_and_checksum_from_v3_shape(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "config.json"
        f.write_text("{}")

        bundle_files = [{
            "filename": "config.json",
            "location": "/sdcard",
            "checksum": {"value": "abc123", "type": "sha512"},
        }]
        local_hashes = {f: "abc123"}

        svc._validate_bundle_files_match(bundle_files, local_hashes, tmp_path)

    def test_raises_on_hash_mismatch(self, mocker, tmp_path):
        svc = _service(mocker)
        f = tmp_path / "config.json"
        f.write_text("{}")

        bundle_files = [{
            "filename": "config.json",
            "location": "/sdcard",
            "checksum": {"value": "remote-hash"},
        }]
        local_hashes = {f: "local-hash"}

        with pytest.raises(ValueError, match="content changed"):
            svc._validate_bundle_files_match(bundle_files, local_hashes, tmp_path)

    def test_raises_on_missing_local_file(self, mocker, tmp_path):
        svc = _service(mocker)

        bundle_files = [{
            "filename": "missing.json",
            "location": "/sdcard",
            "checksum": {"value": "h"},
        }]

        with pytest.raises(ValueError, match="not found locally"):
            svc._validate_bundle_files_match(bundle_files, {}, tmp_path)


class TestUrls:
    def test_list_uses_v3(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [], "links": {}})

        svc.get_all_app_bundles_for_app("app-id")

        url = svc.client.get.call_args[0][0]
        assert url == "https://api.xrdm.app/api/v3/apps/app-id/app-bundles?per_page=20"

    def test_list_with_status_filter(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [], "links": {}})

        svc.get_all_app_bundles_for_app("app-id", status="pending")

        url = svc.client.get.call_args[0][0]
        assert "status=pending" in url

    def test_detail_uses_v3(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"id": "b1"})

        svc.get_app_bundle_detail("b1")

        svc.client.get.assert_called_once_with(
            "https://api.xrdm.app/api/v3/app-bundles/b1",
            headers=svc.headers,
        )

    def test_update_label_uses_patch(self, mocker):
        svc = _service(mocker)
        svc.client.patch.return_value = make_response({"id": "b1", "label": "new"})

        svc.update_app_bundle_label("b1", "new")

        svc.client.patch.assert_called_once_with(
            "https://api.xrdm.app/api/v3/app-bundles/b1",
            json={"label": "new"},
            headers=svc.headers,
        )


def _setup_bundle_scan(mocker, tmp_path, file_names=("config.json", "data.bin")):
    """Build a folder with files and patch _scan_folder to return synthetic hashes."""
    folder = tmp_path / "bundle"
    folder.mkdir()
    apk = tmp_path / "build.apk"
    apk.write_bytes(b"apk-content")

    file_hashes = {}
    for i, name in enumerate(file_names):
        f = folder / name
        f.write_bytes(f"file{i}".encode())
        file_hashes[f] = f"file{i}-sha512"

    return folder, apk, file_hashes


class TestBundleUploadDedup:
    """upload_app_bundle is the headline customer feature — verify the dedup branches."""

    def test_full_dedup_existing_build_and_files(self, mocker, tmp_path):
        """When build AND all files exist, no APK or file uploads happen."""
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path)
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "build-sha256", file_hashes))

        fake_apps_service = mocker.MagicMock()
        fake_apps_service.get_versions_by_sha256.return_value = [{"id": "existing-build"}]
        # All files exist
        existing_files = [
            {"id": f"file-{i}", "filename": path.name,
             "checksum": {"value": h, "type": "sha512"}}
            for i, (path, h) in enumerate(file_hashes.items())
        ]
        fake_apps_service.get_files_by_sha512.return_value = existing_files
        mocker.patch("abxr.app_bundles.AppsService", return_value=fake_apps_service)

        fake_files_service = mocker.MagicMock()
        mocker.patch("abxr.app_bundles.FilesService", return_value=fake_files_service)

        svc.create_app_bundle_from_existing = mocker.MagicMock(
            return_value={"id": "bundle-1"}
        )
        svc.finalize_app_bundle = mocker.MagicMock(
            return_value={"id": "bundle-1", "status": "processing"}
        )

        result = svc.upload_app_bundle(
            "app-1", str(folder), "1.0.0", "notes",
            silent=True, apk_path=str(apk),
        )

        assert result == {"id": "bundle-1", "status": "processing"}
        # No APK upload (build was reused)
        fake_apps_service.upload_file.assert_not_called()
        # No file uploads (all files were reused)
        fake_files_service.upload_file.assert_not_called()
        svc.create_app_bundle_from_existing.assert_called_once()
        svc.finalize_app_bundle.assert_called_once_with("bundle-1")

    def test_no_dedup_uploads_build_and_all_files(self, mocker, tmp_path):
        """When nothing exists, APK uploads, all files upload, bundle finalizes."""
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path)
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "build-sha256", file_hashes))

        fake_apps_service = mocker.MagicMock()
        fake_apps_service.get_versions_by_sha256.return_value = []
        fake_apps_service.get_files_by_sha512.return_value = []
        fake_apps_service.upload_file.return_value = {
            "id": "v1", "appBundleId": "bundle-2",
        }
        mocker.patch("abxr.app_bundles.AppsService", return_value=fake_apps_service)

        fake_files_service = mocker.MagicMock()
        mocker.patch("abxr.app_bundles.FilesService", return_value=fake_files_service)

        svc.finalize_app_bundle = mocker.MagicMock(
            return_value={"id": "bundle-2", "status": "processing"}
        )

        result = svc.upload_app_bundle(
            "app-1", str(folder), "1.0.0", "notes",
            silent=True, apk_path=str(apk),
        )

        assert result == {"id": "bundle-2", "status": "processing"}
        # APK was uploaded with app_build_type='app-bundle'
        fake_apps_service.upload_file.assert_called_once()
        assert fake_apps_service.upload_file.call_args.kwargs["app_build_type"] == "app-bundle"
        # Every file was uploaded
        assert fake_files_service.upload_file.call_count == len(file_hashes)

    def test_partial_dedup_new_build_with_existing_files(self, mocker, tmp_path):
        """New build but some existing files: uploads APK + new files only, adds existing by ref."""
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path,
                                                     file_names=("a.txt", "b.txt", "c.txt"))
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "build-sha256", file_hashes))

        paths = list(file_hashes.keys())
        existing_file_path = paths[0]
        existing_hash = file_hashes[existing_file_path]

        fake_apps_service = mocker.MagicMock()
        fake_apps_service.get_versions_by_sha256.return_value = []  # new build
        fake_apps_service.get_files_by_sha512.return_value = [{
            "id": "file-existing",
            "filename": existing_file_path.name,
            "checksum": {"value": existing_hash, "type": "sha512"},
        }]
        fake_apps_service.upload_file.return_value = {
            "id": "v1", "appBundleId": "bundle-3",
        }
        mocker.patch("abxr.app_bundles.AppsService", return_value=fake_apps_service)

        fake_files_service = mocker.MagicMock()
        mocker.patch("abxr.app_bundles.FilesService", return_value=fake_files_service)

        svc.add_files_to_app_bundle = mocker.MagicMock(return_value={"id": "bundle-3"})
        svc.finalize_app_bundle = mocker.MagicMock(
            return_value={"id": "bundle-3", "status": "processing"}
        )

        svc.upload_app_bundle(
            "app-1", str(folder), "1.0.0", "notes",
            silent=True, apk_path=str(apk),
        )

        # APK uploaded (new build)
        fake_apps_service.upload_file.assert_called_once()
        # One existing file added by reference
        svc.add_files_to_app_bundle.assert_called_once()
        added_files = svc.add_files_to_app_bundle.call_args[0][1]
        assert len(added_files) == 1
        assert added_files[0]["fileId"] == "file-existing"
        # Two new files uploaded (b.txt, c.txt)
        assert fake_files_service.upload_file.call_count == 2


class TestQueryExistingFilesByHash:
    """Direct test of the v3 checksum.value + filename access that this PR changed."""

    def test_matches_by_hash_and_filename(self, mocker, tmp_path):
        svc = _service(mocker)

        p1 = tmp_path / "a.json"
        p2 = tmp_path / "b.json"
        p1.touch()
        p2.touch()
        file_hashes = {p1: "hash-a", p2: "hash-b"}

        fake_apps_service = mocker.MagicMock()
        fake_apps_service.get_files_by_sha512.return_value = [
            {"id": "f1", "filename": "a.json",
             "checksum": {"value": "hash-a", "type": "sha512"}},
            {"id": "f2", "filename": "wrong-name.json",  # hash matches but name doesn't
             "checksum": {"value": "hash-b", "type": "sha512"}},
        ]
        mocker.patch("abxr.app_bundles.AppsService", return_value=fake_apps_service)

        result = svc._query_existing_files_by_hash("app-1", file_hashes, silent=True)

        # Only a.json matched (b.json's hash matched but filename didn't)
        assert p1 in result
        assert p2 not in result
        assert result[p1]["id"] == "f1"

    def test_empty_input_short_circuits(self, mocker):
        svc = _service(mocker)
        result = svc._query_existing_files_by_hash("app-1", {}, silent=True)
        assert result == {}


class TestBundleResume:
    """resume_app_bundle uses v3 'filename' and 'checksum.value' fields directly."""

    def test_happy_path_resumes_pending_bundle(self, mocker, tmp_path):
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path,
                                                     file_names=("config.json",))
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "build-sha256", file_hashes))

        svc.get_app_bundle_detail = mocker.MagicMock(return_value={
            "id": "bundle-1",
            "status": "pending",
            "appBuild": {"checksum": {"value": "build-sha256", "type": "sha256"}},
        })
        # No existing files yet -> all need upload
        svc.get_all_files_for_app_bundle = mocker.MagicMock(return_value=[])

        fake_files_service = mocker.MagicMock()
        mocker.patch("abxr.app_bundles.FilesService", return_value=fake_files_service)
        svc.finalize_app_bundle = mocker.MagicMock(
            return_value={"id": "bundle-1", "status": "processing"}
        )

        result = svc.resume_app_bundle(
            "bundle-1", str(apk), str(folder), silent=True,
        )

        assert result == {"id": "bundle-1", "status": "processing"}
        # Single missing file gets uploaded
        assert fake_files_service.upload_file.call_count == 1

    def test_rejects_non_pending_bundle(self, mocker, tmp_path):
        svc = _service(mocker)
        svc.get_app_bundle_detail = mocker.MagicMock(return_value={
            "id": "bundle-1", "status": "available",
        })

        with pytest.raises(ValueError, match="Only 'pending' bundles can be resumed"):
            svc.resume_app_bundle(
                "bundle-1", "/fake.apk", str(tmp_path), silent=True,
            )

    def test_rejects_build_hash_mismatch(self, mocker, tmp_path):
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path)
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "local-build-hash", file_hashes))

        svc.get_app_bundle_detail = mocker.MagicMock(return_value={
            "id": "bundle-1",
            "status": "pending",
            "appBuild": {"checksum": {"value": "different-remote-hash", "type": "sha256"}},
        })

        with pytest.raises(ValueError, match="build file mismatch"):
            svc.resume_app_bundle(
                "bundle-1", str(apk), str(folder), silent=True,
            )

    def test_skips_already_uploaded_files(self, mocker, tmp_path):
        """Files whose hash matches an existing bundle file should not be re-uploaded."""
        folder, apk, file_hashes = _setup_bundle_scan(mocker, tmp_path,
                                                     file_names=("a.txt", "b.txt"))
        svc = _service(mocker)

        mocker.patch.object(svc, "_scan_folder",
                            return_value=(folder, apk, "build-hash", file_hashes))

        paths = list(file_hashes.keys())
        # a.txt is already uploaded with matching hash + location
        existing_bundle_files = [{
            "filename": paths[0].name,
            "location": "/sdcard",
            "checksum": {"value": file_hashes[paths[0]], "type": "sha512"},
        }]

        svc.get_app_bundle_detail = mocker.MagicMock(return_value={
            "id": "bundle-1",
            "status": "pending",
            "appBuild": {"checksum": {"value": "build-hash"}},
        })
        svc.get_all_files_for_app_bundle = mocker.MagicMock(return_value=existing_bundle_files)

        fake_files_service = mocker.MagicMock()
        mocker.patch("abxr.app_bundles.FilesService", return_value=fake_files_service)
        svc.finalize_app_bundle = mocker.MagicMock(return_value={"id": "bundle-1"})

        svc.resume_app_bundle("bundle-1", str(apk), str(folder), silent=True)

        # Only b.txt should upload (a.txt was already there)
        assert fake_files_service.upload_file.call_count == 1
        uploaded_path = fake_files_service.upload_file.call_args[0][0]
        assert uploaded_path.endswith("b.txt")



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import warnings
warnings.filterwarnings("ignore", message="urllib3 v2 only supports OpenSSL")

import argparse
import os
from requests import HTTPError

from abxr.version import version
from abxr.formats import DataOutputFormats

from abxr.apps import Commands as AppCommands, CommandHandler as AppsCommandHandler
from abxr.app_bundles import Commands as AppBundlesCommands, CommandHandler as AppBundlesCommandHandler
from abxr.files import Commands as FileCommands, CommandHandler as FilesCommandHandler
from abxr.devices import Commands as DeviceCommands, CommandHandler as DevicesCommandHandler
from abxr.system_apps import Commands as SystemAppCommands, CommandHandler as SystemAppsCommandHandler
from abxr.org import Commands as OrgCommands, CommandHandler as OrgCommandHandler
from abxr.audit_logs import Commands as AuditLogsCommands, CommandHandler as AuditLogsCommandHandler
from abxr.groups import Commands as GroupsCommands, CommandHandler as GroupsCommandHandler
from abxr.tags import Commands as TagsCommands, CommandHandler as TagsCommandHandler
from abxr.users import Commands as UsersCommands, CommandHandler as UsersCommandHandler
from abxr.videos import Commands as VideosCommands, CommandHandler as VideosCommandHandler

ABXR_API_URL = os.environ.get("ABXR_API_URL", "https://api.xrdm.app")
ABXR_API_TOKEN = os.environ.get("ABXR_API_TOKEN") or os.environ.get("ARBORXR_ACCESS_TOKEN")


def main():
    parser = argparse.ArgumentParser(description=f'%(prog)s {version}')
    parser.add_argument("-u", "--url", help="API Base URL", type=str, default=ABXR_API_URL)
    parser.add_argument("-t", "--token", help="API Token", type=str, default=ABXR_API_TOKEN)
    parser.add_argument("-f", "--format", help="Data Output format", type=str, choices=[DataOutputFormats.JSON.value, DataOutputFormats.YAML.value], default=DataOutputFormats.YAML.value)
    parser.add_argument("-s", "--silent", help="Hides progress bars or other messages not to interfere with return value processing from stdout", action="store_true")
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s {version}')

    subparsers = parser.add_subparsers(dest="command", help="Sub-command help")

    # Org
    org_parser = subparsers.add_parser("org", help="Organization command")
    org_subparsers = org_parser.add_subparsers(dest="org_command", help="Organization command help")

    org_info_parser = org_subparsers.add_parser(OrgCommands.INFO.value, help="Get organization info connected to API token")

    # Apps
    apps_parser = subparsers.add_parser("apps", help="Apps command")
    apps_subparsers = apps_parser.add_subparsers(dest="apps_command", help="Apps command help")

    # List All Apps
    apps_list_parser = apps_subparsers.add_parser(AppCommands.LIST.value, help="List apps")

    # Detail of App
    app_detail_parser = apps_subparsers.add_parser(AppCommands.DETAILS.value, help="Get the details of an app")
    app_detail_parser.add_argument("app_id", help="ID of the app", type=str)

    # Versions of an App
    versions_list_parser = apps_subparsers.add_parser(AppCommands.VERSION_LIST.value, help="List versions of an app")
    versions_list_parser.add_argument("app_id", help="ID of the app", type=str)

    # List Release Channels
    release_channels_list_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNELS_LIST.value, help="List release channels of an app")
    release_channels_list_parser.add_argument("app_id", help="ID of the app", type=str)

    # Detail of Release Channel
    release_channel_detail_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNEL_DETAILS.value, help="Detail of a release channel")
    release_channel_detail_parser.add_argument("app_id", help="ID of the app", type=str)
    release_channel_detail_parser.add_argument("--release_channel_id", help="ID of the release channel", type=str, required=True)

    # Set Version for Release Channel
    release_channel_set_version_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNEL_SET_VERSION.value, help="Set version for a release channel")
    release_channel_set_version_parser.add_argument("app_id", help="ID of the app", type=str)
    release_channel_set_version_parser.add_argument("--release_channel_id", help="ID of the release channel", type=str, required=True)
    version_group = release_channel_set_version_parser.add_mutually_exclusive_group(required=True)
    version_group.add_argument("--version_id", help="ID of the version (app build)", type=str)
    version_group.add_argument("--bundle_id", help="ID of the app bundle", type=str)

    # Upload and Create Version
    create_version_parser = apps_subparsers.add_parser(AppCommands.UPLOAD.value, help="Upload a new version of an app")
    create_version_parser.add_argument("app_id", help="ID of the app", type=str)
    create_version_parser.add_argument("filename", help="Local path of the APK/ZIP (apk+obb) to upload", type=str)
    create_version_parser.add_argument("--version_number", help="Version Number (Uploaded APK can override this value)", type=str)
    create_version_parser.add_argument("-n", "--notes", help="Release Notes", type=str)
    create_version_parser.add_argument("-w", "--wait", help="Wait for the upload to complete", action="store_true")
    create_version_parser.add_argument("--wait_time", help="Maximum wait time in seconds for the upload to complete processing", type=int, default=60)

    # Release channel options (mutually exclusive)
    release_channel_group = create_version_parser.add_mutually_exclusive_group()
    release_channel_group.add_argument("--release-channel-id", help="ID of existing release channel to upload to", type=str, dest="release_channel_id")
    release_channel_group.add_argument("--new-release-channel-title", help="Title for a new release channel to create", type=str, dest="new_release_channel_title")

    # Sharing Apps
    share_parser = apps_subparsers.add_parser(AppCommands.SHARE.value, help="Share an app")
    share_parser.add_argument("app_id", help="ID of the app", type=str)
    share_parser.add_argument("--release_channel_id", help="ID of the release channel to share", type=str, required=True)
    share_parser.add_argument("--organization_slug", help="Slug of the organization to share with", type=str, required=True)

    # Revoke Sharing
    revoke_share_parser = apps_subparsers.add_parser(AppCommands.REVOKE_SHARE.value, help="Revoke sharing of an app")
    revoke_share_parser.add_argument("app_id", help="ID of the app", type=str)
    revoke_share_parser.add_argument("--release_channel_id", help="ID of the release channel to revoke", type=str, required=True)
    revoke_share_parser.add_argument("--organization_slug", help="Slug of the organization to revoke from", type=str, required=True)

    ## Audit Logs
    audit_logs_parser = subparsers.add_parser("audit_logs", help="Audit Logs command")
    audit_logs_subparsers = audit_logs_parser.add_subparsers(dest="audit_logs_command", help="Audit Logs command help")

    # List Audit Logs
    audit_logs_list_parser = audit_logs_subparsers.add_parser(AuditLogsCommands.LIST.value, help="List all audit logs")
    audit_logs_list_parser.add_argument("--search", help="Search term to filter audit logs. Searches across description, user name, user email, resource title, and IP address.", type=str)
    audit_logs_list_parser.add_argument("--start_time", help="Filter audit logs to show only entries created at or after this timestamp (ISO 8601 Zulu format with millisecond precision).", type=str)
    audit_logs_list_parser.add_argument("--end_time", help="Filter audit logs to show only entries created at or before this timestamp (ISO 8601 Zulu format with millisecond precision).", type=str)

    ## Groups
    groups_parser = subparsers.add_parser("groups", help="Groups command")
    groups_subparsers = groups_parser.add_subparsers(dest="groups_command", help="Groups command help")

    # List Groups
    groups_list_parser = groups_subparsers.add_parser(GroupsCommands.LIST.value, help="List all groups")

    # Create a new Group
    groups_add_parser = groups_subparsers.add_parser(GroupsCommands.ADD.value, help="Create a new group")
    groups_add_parser.add_argument("name", help="Name of the group", type=str)
    groups_add_parser.add_argument("--parent_group_id", help="Parent ID of the group to attach to", type=str)

    # Group Detail
    groups_detail_parser = groups_subparsers.add_parser(GroupsCommands.DETAILS.value, help="Get details of a group")
    groups_detail_parser.add_argument("group_id", help="ID of the group", type=str)

    # Group Update
    groups_update_parser = groups_subparsers.add_parser(GroupsCommands.UPDATE.value, help="Update a group")
    groups_update_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_update_parser.add_argument("--name", help="New name of the group", type=str)
    groups_update_parser.add_argument("--parent_group_id", help="New parent ID of the group", type=str)

    # Group Delete
    groups_delete_parser = groups_subparsers.add_parser(GroupsCommands.DELETE.value, help="Delete a group")
    groups_delete_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_delete_parser.add_argument("--replacement_group_id", help="ID of replacement group to move child groups to (only allowed for unconfigured groups)", type=str)

    # Group Configure
    groups_configure_parser = groups_subparsers.add_parser(GroupsCommands.CONFIGURE.value, help="Configure a group (convert from unconfigured to configured state).")
    groups_configure_parser.add_argument("group_id", help="ID of the group", type=str)

    # Group Duplicate
    groups_duplicate_parser = groups_subparsers.add_parser(GroupsCommands.DUPLICATE.value, help="Duplicate a group")
    groups_duplicate_parser.add_argument("group_id", help="ID of the group to duplicate", type=str)
    groups_duplicate_parser.add_argument("--name", help="New name for the duplicated group", type=str, required=True)

    # List Release Channels assigned to a Group
    groups_release_channels_list_parser = groups_subparsers.add_parser(GroupsCommands.RELEASE_CHANNELS_LIST.value, help="List release channels assigned to a group")
    groups_release_channels_list_parser.add_argument("group_id", help="ID of the group", type=str)

    # Assign a Release Channel to a Group
    groups_release_channel_add_parser = groups_subparsers.add_parser(GroupsCommands.RELEASE_CHANNEL_ADD.value, help="Assign a release channel to a group")
    groups_release_channel_add_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_release_channel_add_parser.add_argument("--release_channel_id", help="ID of the release channel to assign", type=str, required=True)

    # Remove a Release Channel from a Group
    groups_release_channel_remove_parser = groups_subparsers.add_parser(GroupsCommands.RELEASE_CHANNEL_REMOVE.value, help="Remove a release channel from a group")
    groups_release_channel_remove_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_release_channel_remove_parser.add_argument("--release_channel_id", help="ID of the release channel to remove", type=str, required=True)

    # Add a File to a Group
    groups_files_add_parser = groups_subparsers.add_parser(GroupsCommands.FILES_ADD.value, help="Add a file to a group")
    groups_files_add_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_files_add_parser.add_argument("--file_id", help="ID of the file to add", type=str, required=True)

    # Remove a File from a Group
    groups_files_remove_parser = groups_subparsers.add_parser(GroupsCommands.FILES_REMOVE.value, help="Remove a file from a group")
    groups_files_remove_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_files_remove_parser.add_argument("--file_id", help="ID of the file to remove", type=str, required=True)

    # Add a Video to a Group
    groups_video_add_parser = groups_subparsers.add_parser(GroupsCommands.VIDEO_ADD.value, help="Add a video to a group")
    groups_video_add_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_video_add_parser.add_argument("--video_id", help="ID of the video to add", type=str, required=True)

    # Remove a Video from a Group
    groups_video_remove_parser = groups_subparsers.add_parser(GroupsCommands.VIDEO_REMOVE.value, help="Remove a video from a group")
    groups_video_remove_parser.add_argument("group_id", help="ID of the group", type=str)
    groups_video_remove_parser.add_argument("--video_id", help="ID of the video to remove", type=str, required=True)

    # Get Group Hierarchy
    groups_hierarchy_detail_parser = groups_subparsers.add_parser(GroupsCommands.HIERARCHY_DETAIL.value, help="Get the full group hierarchy")

    

    ## Files
    files_parser = subparsers.add_parser("files", help="Files command")
    files_subparsers = files_parser.add_subparsers(dest="files_command", help="Files command help")

    # List All Files
    files_list_parser = files_subparsers.add_parser(FileCommands.LIST.value, help="List all files in content library")

    # Detail of File
    file_detail_parser = files_subparsers.add_parser(FileCommands.DETAILS.value, help="Detail of an uploaded file")
    file_detail_parser.add_argument("file_id", help="ID of the file", type=str)

    # Upload a file
    upload_file_parser = files_subparsers.add_parser(FileCommands.UPLOAD.value, help="Upload a file")
    upload_file_parser.add_argument("filename", help="Local path of the file to upload", type=str)
    upload_file_parser.add_argument("--device-path", "--device_path", help="Desired path of the file on the device", type=str, required=True)

    # List All Files Assigned to a Device
    files_device_list_parser = files_subparsers.add_parser(FileCommands.DEVICE_LIST.value, help="List files assigned to a device")
    files_device_list_parser.add_argument("device_id", help="ID of the device", type=str)

    # Assign a File to a Device
    files_device_assign_parser = files_subparsers.add_parser(FileCommands.DEVICE_ASSIGN.value, help="Assign a file to a device")
    files_device_assign_parser.add_argument("device_id", help="ID of the device", type=str)
    files_device_assign_parser.add_argument("--file_id", help="ID of the file to assign", type=str, required=True)

    # Remove a File from a Device
    files_device_remove_parser = files_subparsers.add_parser(FileCommands.DEVICE_REMOVE.value, help="Remove a file from a device")
    files_device_remove_parser.add_argument("device_id", help="ID of the device", type=str)
    files_device_remove_parser.add_argument("--file_id", help="ID of the file to remove", type=str, required=True)

    # Assign a File to a Device Group
    files_device_group_assign_parser = files_subparsers.add_parser(FileCommands.GROUP_ASSIGN.value, help="Assign a file to a group")
    files_device_group_assign_parser.add_argument("group_id", help="ID of the device group", type=str)
    files_device_group_assign_parser.add_argument("--file_id", help="ID of the file to assign", type=str, required=True)

    # Remove a File from a Device Group
    files_device_group_remove_parser = files_subparsers.add_parser(FileCommands.GROUP_REMOVE.value, help="Remove a file from a group")
    files_device_group_remove_parser.add_argument("group_id", help="ID of the device group", type=str)
    files_device_group_remove_parser.add_argument("--file_id", help="ID of the file to remove", type=str, required=True)

    # Devices
    devices_parser = subparsers.add_parser("devices", help="Devices command")
    devices_subparsers = devices_parser.add_subparsers(dest="devices_command", help="Devices command help")

    # List All Devices
    devices_list_parser = devices_subparsers.add_parser(DeviceCommands.LIST.value, help="List devices")

    # Detail of Device
    device_detail_parser = devices_subparsers.add_parser(DeviceCommands.DETAILS.value, help="Detail of a device")
    device_detail_parser.add_argument("device_id", help="ID of the device", type=str)

    # Launch App on Device
    launch_app_parser = devices_subparsers.add_parser(DeviceCommands.LAUNCH_APP.value, help="Launch an app on a device")
    launch_app_parser.add_argument("device_id", help="ID of the device", type=str)
    launch_app_parser.add_argument("--app_id", help="ID of the app", type=str, required=True)

    # Reboot Device
    reboot_device_parser = devices_subparsers.add_parser(DeviceCommands.REBOOT.value, help="Reboot a device")
    reboot_device_parser.add_argument("device_id", help="ID of the device", type=str)

    # System Apps
    system_apps_parser = subparsers.add_parser("system_apps", help="System Apps command")
    system_apps_subparsers = system_apps_parser.add_subparsers(dest="system_apps_command", help="System Apps command help")

    # List All System App Versions
    system_apps_list_parser = system_apps_subparsers.add_parser(SystemAppCommands.VERSIONS_LIST.value, help="List system app versions")
    system_apps_list_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)

    # Upload System App
    upload_system_app_parser = system_apps_subparsers.add_parser(SystemAppCommands.UPLOAD.value, help="Upload a system app")
    upload_system_app_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)
    upload_system_app_parser.add_argument("filename", help="Local path of the APK to upload", type=str)
    upload_system_app_parser.add_argument("--version_number", help="Version Number (APK can override this value; required semver for 'os' app type)", type=str)
    upload_system_app_parser.add_argument("--version_code", help="Version Code (required for 'os' app type only)", type=int)
    upload_system_app_parser.add_argument("-n", "--notes", help="Release Notes", type=str)
    upload_system_app_parser.add_argument("--app_compatibility_name", help="App compatibility name (e.g: armeabi-v7a, PICO 4 Enterprise) (required)", type=str, required=True)

    # List Release Channels for System App
    release_channels_list_system_parser = system_apps_subparsers.add_parser(SystemAppCommands.RELEASE_CHANNELS_LIST.value, help="List release channels for a system app")
    release_channels_list_system_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)

    # Detail of Release Channel for System App
    release_channel_detail_system_parser = system_apps_subparsers.add_parser(SystemAppCommands.RELEASE_CHANNEL_DETAILS.value, help="Detail of a release channel for a system app")
    release_channel_detail_system_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)
    release_channel_detail_system_parser.add_argument("--release_channel_id", help="ID of the release channel", type=str, required=True)

    # List App Compatibilities for System App
    app_compatibilities_list_system_parser = system_apps_subparsers.add_parser(SystemAppCommands.APP_COMPATIBILITIES.value, help="List app compatibilities for a system app")
    app_compatibilities_list_system_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)

    # Detail of App Compatibility for System App
    app_compatibility_detail_system_parser = system_apps_subparsers.add_parser(SystemAppCommands.APP_COMPATIBILITY_DETAILS.value, help="Detail of an app compatibility for a system app")
    app_compatibility_detail_system_parser.add_argument("app_type", help="Type of the system app (e.g., 'client', 'home')", type=str)
    app_compatibility_detail_system_parser.add_argument("--app_compatibility_id", help="ID of the app compatibility", type=str, required=True)

    # Tags
    tags_parser = subparsers.add_parser("tags", help="Tags command")
    tags_subparsers = tags_parser.add_subparsers(dest="tags_command", help="Tags command help")

    # List Tags
    tags_list_parser = tags_subparsers.add_parser(TagsCommands.LIST.value, help="List all tags")

    # Create Tag
    tags_create_parser = tags_subparsers.add_parser(TagsCommands.CREATE.value, help="Create a new tag")
    tags_create_parser.add_argument("--name", help="Name of the new tag", type=str, required=True)
   
    # Tag Details
    tags_detail_parser = tags_subparsers.add_parser(TagsCommands.DETAIL.value, help="Get details of a tag")
    tags_detail_parser.add_argument("tag_id", help="ID of the tag", type=str)
   
    # Update Tag
    tags_update_parser = tags_subparsers.add_parser(TagsCommands.UPDATE.value, help="Update a tag")
    tags_update_parser.add_argument("tag_id", help="ID of the tag", type=str)
    tags_update_parser.add_argument("--name", help="New name for the tag", type=str, required=True)
   
    # Delete Tag
    tags_delete_parser = tags_subparsers.add_parser(TagsCommands.DELETE.value, help="Delete a tag")
    tags_delete_parser.add_argument("tag_id", help="ID of the tag", type=str)

    # Users
    users_parser = subparsers.add_parser("users", help="Users command")
    users_subparsers = users_parser.add_subparsers(dest="users_command", help="Users command help")

    # List Users
    users_list_parser = users_subparsers.add_parser(UsersCommands.LIST.value, help="List all users")

    # Create User
    users_create_parser = users_subparsers.add_parser(UsersCommands.CREATE.value, help="Create a new user")
    users_create_parser.add_argument("--first_name", help="First name of the new user", type=str, required=True)
    users_create_parser.add_argument("--last_name", help="Last name of the new user", type=str, required=True)
    users_create_parser.add_argument("--email", help="Email of the new user", type=str, required=True)
    users_create_parser.add_argument("--org_role_id", help="Organization role ID", type=str)

    # User Details
    users_detail_parser = users_subparsers.add_parser(UsersCommands.DETAILS.value, help="Get details of a user")
    users_detail_parser.add_argument("user_id", help="ID of the user", type=str)

    # Update User
    users_update_parser = users_subparsers.add_parser(UsersCommands.UPDATE.value, help="Update a user")
    users_update_parser.add_argument("user_id", help="ID of the user", type=str)
    users_update_parser.add_argument("--first_name", help="New first name for the user", type=str)
    users_update_parser.add_argument("--last_name", help="New last name for the user", type=str)

    # Delete User
    users_delete_parser = users_subparsers.add_parser(UsersCommands.DELETE.value, help="Delete a user")
    users_delete_parser.add_argument("user_id", help="ID of the user", type=str)

    # Videos
    videos_parser = subparsers.add_parser("videos", help="Videos command")
    videos_subparsers = videos_parser.add_subparsers(dest="videos_command", help="Videos command help")

    # List All Videos
    videos_list_parser = videos_subparsers.add_parser(VideosCommands.LIST.value, help="List all videos")

    # Video Detail
    videos_detail_parser = videos_subparsers.add_parser(VideosCommands.DETAILS.value, help="Get details of a video")
    videos_detail_parser.add_argument("video_id", help="ID of the video", type=str)

    # Update Video
    videos_update_parser = videos_subparsers.add_parser(VideosCommands.UPDATE, help="Update the properties of a video")
    videos_update_parser.add_argument("video_id", help="ID of the video", type=str)
    videos_update_parser.add_argument("--name", help="Name of the video", type=str)
    videos_update_parser.add_argument("--description", help="Description of the video", type=str)
    videos_update_parser.add_argument("--video_type", help="Type of the video", type=str, choices=["ThreeSixty", "OneEighty", "TwoDimensional"])
    videos_update_parser.add_argument("--video_mapping", help="Mapping of the video", type=str, choices=["EQUIRECTANGULAR", "CUBEMAP"])
    videos_update_parser.add_argument("--video_display", help="Display of the video", type=str, choices=["Stereoscopic", "Monoscopic"])
    videos_update_parser.add_argument("--video_packing", help="Packing of the video", type=str, choices=["TOP_BOTTOM", "LEFT_RIGHT"])
    videos_update_parser.add_argument("--audio_encoding", help="Audio Encoding", type=str, choices=["UNKNOWN", "MONO", "STEREO", "TBE_8", "TBE_8_2", "TBE_6", "TBE_6_2", "TBE_4", "TBE_4_2", "TBE_8_PAIR_0", "TBE_8_PAIR_1", "TBE_8_PAIR_2", "TBE_8_PAIR_3", "TBE_CHANNEL_0", "TBE_CHANNEL_1", "TBE_CHANNEL_2", "TBE_CHANNEL_3", "TBE_CHANNEL_4", "TBE_CHANNEL_5", "TBE_CHANNEL_6", "TBE_CHANNEL_7", "HEADLOCKED_STEREO", "HEADLOCKED_CHANNEL_0", "HEADLOCKED_CHANNEL_1", "AMBIX_4", "AMBIX_4_2", "AMBIX_9", "AMBIX_9_2", "AMBIX_16", "AMBIX_16_2"])
    videos_update_parser.add_argument("--tags", help="Tags to add to the video", type=list)

    # Upload New Video
    videos_upload_parser = videos_subparsers.add_parser(VideosCommands.UPLOAD.value, help="Upload a new video")
    videos_upload_parser.add_argument("filename", help="Local path of the video file to upload", type=str)
    videos_upload_parser.add_argument("--video_type", help="Type of the video", type=str, choices=["ThreeSixty", "OneEighty", "TwoDimensional"], required=True)
    videos_upload_parser.add_argument("--video_mapping", help="Mapping of the video", type=str, choices=["EQUIRECTANGULAR", "CUBEMAP"])
    videos_upload_parser.add_argument("--video_display", help="Display of the video", type=str, choices=["Stereoscopic", "Monoscopic"])
    videos_upload_parser.add_argument("--video_packing", help="Packing of the video", type=str, choices=["TOP_BOTTOM", "LEFT_RIGHT"])
    videos_upload_parser.add_argument("--audio_encoding", help="Audio Encoding", type=str, choices=["UNKNOWN", "MONO", "STEREO", "TBE_8", "TBE_8_2", "TBE_6", "TBE_6_2", "TBE_4", "TBE_4_2", "TBE_8_PAIR_0", "TBE_8_PAIR_1", "TBE_8_PAIR_2", "TBE_8_PAIR_3", "TBE_CHANNEL_0", "TBE_CHANNEL_1", "TBE_CHANNEL_2", "TBE_CHANNEL_3", "TBE_CHANNEL_4", "TBE_CHANNEL_5", "TBE_CHANNEL_6", "TBE_CHANNEL_7", "HEADLOCKED_STEREO", "HEADLOCKED_CHANNEL_0", "HEADLOCKED_CHANNEL_1", "AMBIX_4", "AMBIX_4_2", "AMBIX_9", "AMBIX_9_2", "AMBIX_16", "AMBIX_16_2"])

    # Attach Tags to a video
    videos_attach_tags_parser = videos_subparsers.add_parser(VideosCommands.ATTACH_TAGS.value, help="Attach tags to a video")
    videos_attach_tags_parser.add_argument("video_id", help="ID of the video", type=str)
    videos_attach_tags_parser.add_argument("--tags", help="Tags to add to the video", type=list, required=True)

    # Detach Tags from a video
    videos_detach_tags_parser = videos_subparsers.add_parser(VideosCommands.DETACH_TAGS, help="Detach tags from a video")
    videos_detach_tags_parser.add_argument("video_id", help="ID of the video", type=str)
    videos_detach_tags_parser.add_argument("--tags", help="Tags to remove from the video", type=list, required=True)

    # App Bundles
    app_bundles_parser = subparsers.add_parser("app_bundles", help="App Bundles command")
    app_bundles_subparsers = app_bundles_parser.add_subparsers(dest="app_bundles_command", help="App Bundles command help")

    # Upload an app bundle
    app_bundles_upload_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.UPLOAD.value, help="Upload an app bundle with APK and files (bundle labels are auto-generated)")
    app_bundles_upload_parser.add_argument("app_id", help="ID of the app", type=str)
    app_bundles_upload_parser.add_argument("apk_path", help="Path to APK file", type=str)
    app_bundles_upload_parser.add_argument("bundle_folder", help="Path to folder containing bundle files", type=str)
    app_bundles_upload_parser.add_argument("--version_number", help="Version number (APK can override this value)", type=str)
    app_bundles_upload_parser.add_argument("-n", "--notes", help="Release notes for the bundle", type=str)
    app_bundles_upload_parser.add_argument("--device-path", "--device_path", help="Optional device path relative to /sdcard for bundle files", type=str)
    app_bundles_upload_parser.add_argument("-s", "--silent", help="Suppress progress bars and other output", action="store_true")

    # Release channel options (mutually exclusive)
    bundle_release_channel_group = app_bundles_upload_parser.add_mutually_exclusive_group()
    bundle_release_channel_group.add_argument("--release-channel-id", help="ID of existing release channel to upload to", type=str, dest="release_channel_id")
    bundle_release_channel_group.add_argument("--new-release-channel-title", help="Title for a new release channel to create", type=str, dest="new_release_channel_title")

    # List app bundles for an app
    app_bundles_list_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.LIST.value, help="List app bundles for an app")
    app_bundles_list_parser.add_argument("app_id", help="ID of the app", type=str)
    app_bundles_list_parser.add_argument("--status", help="Filter by status (pending, processing, failed, available)", type=str)

    # Get app bundle details
    app_bundles_details_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.DETAILS.value, help="Get details of an app bundle")
    app_bundles_details_parser.add_argument("app_bundle_id", help="ID of the app bundle", type=str)

    # Resume a failed or interrupted app bundle upload
    app_bundles_resume_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.RESUME.value, help="Resume a failed or interrupted bundle upload")
    app_bundles_resume_parser.add_argument("bundle_id", help="ID of the bundle to resume", type=str)
    app_bundles_resume_parser.add_argument("apk_path", help="Path to APK file", type=str)
    app_bundles_resume_parser.add_argument("folder_path", help="Path to folder with bundle files", type=str)
    app_bundles_resume_parser.add_argument("--device-path", "--device_path", help="Optional device path relative to /sdcard for bundle files (must match original upload)", type=str)
    app_bundles_resume_parser.add_argument("-s", "--silent", help="Suppress progress bars and other output", action="store_true")

    # Update an app bundle's label
    app_bundles_update_label_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.UPDATE_LABEL.value, help="Update an app bundle's label")
    app_bundles_update_label_parser.add_argument("app_bundle_id", help="ID of the app bundle", type=str)
    label_group = app_bundles_update_label_parser.add_mutually_exclusive_group(required=True)
    label_group.add_argument("--label", help="New label for the bundle (max 60 characters)", type=str)
    label_group.add_argument("--clear", help="Remove the label from the bundle", action="store_true")

    # Create app bundle from existing build
    app_bundles_create_from_build_parser = app_bundles_subparsers.add_parser(AppBundlesCommands.CREATE_FROM_BUILD.value, help="Create an app bundle from an existing build ID and folder of files")
    app_bundles_create_from_build_parser.add_argument("build_id", help="ID of the existing app build/version", type=str)
    app_bundles_create_from_build_parser.add_argument("bundle_folder", help="Path to folder containing bundle files", type=str)
    app_bundles_create_from_build_parser.add_argument("app_id", help="ID of the app (needed for file deduplication)", type=str)
    app_bundles_create_from_build_parser.add_argument("--device-path", "--device_path", help="Optional device path relative to /sdcard for bundle files", type=str)
    app_bundles_create_from_build_parser.add_argument("-s", "--silent", help="Suppress progress bars and other output", action="store_true")

    # Release channel options (mutually exclusive)
    create_from_build_release_channel_group = app_bundles_create_from_build_parser.add_mutually_exclusive_group()
    create_from_build_release_channel_group.add_argument("--release-channel-id", help="ID of existing release channel to associate with bundle", type=str, dest="release_channel_id")
    create_from_build_release_channel_group.add_argument("--new-release-channel-title", help="Title for a new release channel to create", type=str, dest="new_release_channel_title")

    args = parser.parse_args()

    if args.url is None:
        print("API URL is required")
        exit(1)

    if args.token is None:
        print("API Token is required. Please set the ABXR_API_TOKEN environment variable or use the --token command line param.")
        exit(1)

    try:
        if args.command == "apps":
            handler = AppsCommandHandler(args)
            handler.run()

        elif args.command == "audit_logs":
            handler = AuditLogsCommandHandler(args)
            handler.run()

        elif args.command == "files":
            handler = FilesCommandHandler(args)
            handler.run()

        elif args.command == "devices":
            handler = DevicesCommandHandler(args)
            handler.run()

        elif args.command == "groups":
            handler = GroupsCommandHandler(args)
            handler.run()

        elif args.command == "system_apps":
            handler = SystemAppsCommandHandler(args)
            handler.run()

        elif args.command == "org":
            handler = OrgCommandHandler(args)
            handler.run()

        elif args.command == "tags":
            handler = TagsCommandHandler(args)
            handler.run()

        elif args.command == "users":
            handler = UsersCommandHandler(args)
            handler.run()

        elif args.command == "videos":
            handler = VideosCommandHandler(args)
            handler.run()

        elif args.command == "app_bundles":
            handler = AppBundlesCommandHandler(args)
            handler.run()

    except HTTPError as e:
        if e.response is not None and e.response.status_code == 401:
            print("Unauthorized: Invalid API Token.")
            exit(1)
        elif e.response is not None and e.response.status_code == 422:
            try:
                body = e.response.json()
                message = body.get("message", "Validation Error")
                print(f"Validation Error: {message}")
                errors = body.get("errors", {})
                for field, messages in errors.items():
                    for msg in messages:
                        print(f"  {field}: {msg}")
            except (ValueError, KeyError):
                print(f"HTTP Error: {e}")
            exit(1)
        else:
            print(f"HTTP Error: {e}")
            exit(1)
    
    except Exception as e:
        print(f"Error: {e}")
        exit(1)
        

if __name__ == "__main__":
    main()


import pytest

from abxr.api_service import ApiService
from tests.conftest import make_response


class TestUrlNormalization:
    def test_bare_origin(self):
        svc = ApiService("https://api.xrdm.app", "tok")
        assert svc._base_origin == "https://api.xrdm.app"

    def test_trailing_slash_origin(self):
        svc = ApiService("https://api.xrdm.app/", "tok")
        assert svc._base_origin == "https://api.xrdm.app"

    def test_strips_api(self):
        svc = ApiService("https://api.xrdm.app/api", "tok")
        assert svc._base_origin == "https://api.xrdm.app"

    def test_strips_api_v3(self):
        svc = ApiService("https://api.xrdm.app/api/v3", "tok")
        assert svc._base_origin == "https://api.xrdm.app"

    def test_strips_api_v3_with_trailing_slash(self):
        svc = ApiService("https://api.xrdm.app/api/v3/", "tok")
        assert svc._base_origin == "https://api.xrdm.app"

    def test_strips_api_internal(self):
        svc = ApiService("https://api.xrdm.app/api/internal", "tok")
        assert svc._base_origin == "https://api.xrdm.app"


class TestV2Rejection:
    def test_rejects_api_v2(self):
        with pytest.raises(ValueError, match="v2"):
            ApiService("https://api.xrdm.app/api/v2", "tok")

    def test_rejects_api_v2_with_trailing_slash(self):
        with pytest.raises(ValueError, match="v2"):
            ApiService("https://api.xrdm.app/api/v2/", "tok")

    def test_rejects_api_v2_with_extra_path(self):
        with pytest.raises(ValueError, match="v2"):
            ApiService("https://api.xrdm.app/api/v2/apps", "tok")

    def test_error_message_mentions_v3(self):
        with pytest.raises(ValueError, match=r"v3"):
            ApiService("https://api.xrdm.app/api/v2", "tok")


class TestUrlBuilder:
    def test_builds_v3_url(self):
        svc = ApiService("https://api.xrdm.app", "tok")
        assert svc._url("apps") == "https://api.xrdm.app/api/v3/apps"

    def test_joins_multiple_segments(self):
        svc = ApiService("https://api.xrdm.app", "tok")
        assert svc._url("apps", "abc", "versions") == "https://api.xrdm.app/api/v3/apps/abc/versions"

    def test_strips_segment_slashes(self):
        svc = ApiService("https://api.xrdm.app", "tok")
        assert svc._url("/apps/", "abc") == "https://api.xrdm.app/api/v3/apps/abc"


class TestAuthHeaders:
    def test_bearer_token(self):
        svc = ApiService("https://api.xrdm.app", "my-token")
        assert svc.headers["Authorization"] == "Bearer my-token"
        assert svc.headers["Content-Type"] == "application/json"
        assert svc.headers["Accept"] == "application/json"


class TestPagination:
    def test_single_page(self, mocker):
        svc = ApiService("https://api.xrdm.app", "tok")
        svc.client = mocker.MagicMock()
        svc.client.get.return_value = make_response({"data": [{"id": 1}, {"id": 2}]})

        result = svc._get_all_pages("https://api.xrdm.app/api/v3/apps")

        assert result == [{"id": 1}, {"id": 2}]
        assert svc.client.get.call_count == 1

    def test_follows_links_next(self, mocker):
        svc = ApiService("https://api.xrdm.app", "tok")
        svc.client = mocker.MagicMock()
        svc.client.get.side_effect = [
            make_response({
                "data": [{"id": 1}],
                "links": {"next": "https://api.xrdm.app/api/v3/apps?page=2"}
            }),
            make_response({
                "data": [{"id": 2}],
                "links": {"next": "https://api.xrdm.app/api/v3/apps?page=3"}
            }),
            make_response({"data": [{"id": 3}], "links": {"next": None}}),
        ]

        result = svc._get_all_pages("https://api.xrdm.app/api/v3/apps")

        assert result == [{"id": 1}, {"id": 2}, {"id": 3}]
        assert svc.client.get.call_count == 3


class TestParseResponse:
    def test_empty_content_returns_none(self, mocker):
        svc = ApiService("https://api.xrdm.app", "tok")
        response = mocker.MagicMock(content=b"")
        assert svc._parse_response(response) is None

    def test_non_empty_content_returns_json(self, mocker):
        svc = ApiService("https://api.xrdm.app", "tok")
        response = mocker.MagicMock(content=b'{"foo": "bar"}')
        response.json.return_value = {"foo": "bar"}
        assert svc._parse_response(response) == {"foo": "bar"}



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#
from enum import Enum

from abxr.api_service import ApiService
from abxr.formats import DataOutputFormats
from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"

class AuditLogsService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_all_audit_logs(self, search, start_time, end_time):
        url = self._url('audit-logs') + '?per_page=20'

        if search:
            url += f'&search={search}'
        if start_time:
            url += f'&start_time={start_time}'
        if end_time:
            url += f'&end_time={end_time}'

        return self._get_all_pages(url)

class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = AuditLogsService(self.args.url, self.args.token)

    def run(self):
        if self.args.audit_logs_command == Commands.LIST.value:
            audit_logs = self.service.get_all_audit_logs(self.args.search, self.args.start_time, self.args.end_time)
            print_formatted(self.args.format, audit_logs)



"""Tests for MultipartFileS3 chunking math.

The byte-offset math is hard to spot-check when uploads work — corruption
manifests as failed S3 part assembly long after the test would have caught
it. These tests pin part-size calculation, part-count math, and seek/read
offsets by patching the size constants down so we can use tiny files.
"""

import os

import pytest

from abxr.multipart import MultipartFileS3


@pytest.fixture
def tiny_parts(mocker):
    """Shrink MIN_PART_SIZE so tests can use kilobyte-scale files."""
    mocker.patch.object(MultipartFileS3, "MIN_PART_SIZE", 1024)


def _write(tmp_path, name, content):
    f = tmp_path / name
    f.write_bytes(content)
    return f


class TestGetPartSize:
    def test_small_file_uses_min_part_size(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "tiny.bin", b"x" * 100)
        assert MultipartFileS3(str(f)).part_size == 1024

    def test_medium_file_scales_with_size(self, tiny_parts, tmp_path):
        # MAX_PARTS = 1000, so a file of 1000 * 2048 bytes -> part_size = 2048
        f = _write(tmp_path, "med.bin", b"x" * (1000 * 2048))
        assert MultipartFileS3(str(f)).part_size == 2048

    def test_huge_file_clamps_at_max_part_size(self, mocker, tmp_path):
        # Patch both bounds down so we can exercise the upper clamp cheaply.
        mocker.patch.object(MultipartFileS3, "MIN_PART_SIZE", 512)
        mocker.patch.object(MultipartFileS3, "MAX_PART_SIZE", 2048)
        # size // MAX_PARTS would be 4096, but MAX_PART_SIZE clamps it to 2048
        f = _write(tmp_path, "huge.bin", b"x" * (1000 * 4096))
        assert MultipartFileS3(str(f)).part_size == 2048


class TestGetPartNumbers:
    def test_exact_multiple_of_part_size(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "exact.bin", b"x" * (1024 * 3))
        assert MultipartFileS3(str(f)).get_part_numbers() == 3

    def test_one_byte_over_multiple_adds_a_part(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "over.bin", b"x" * (1024 * 3 + 1))
        assert MultipartFileS3(str(f)).get_part_numbers() == 4

    def test_file_smaller_than_part_size_is_one_part(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "small.bin", b"hello")
        assert MultipartFileS3(str(f)).get_part_numbers() == 1


class TestGetPart:
    def test_part_1_reads_from_offset_zero(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "indexed.bin", b"A" * 1024 + b"B" * 1024 + b"C" * 1024)
        assert MultipartFileS3(str(f)).get_part(1) == b"A" * 1024

    def test_middle_part_reads_from_correct_offset(self, tiny_parts, tmp_path):
        f = _write(tmp_path, "indexed.bin", b"A" * 1024 + b"B" * 1024 + b"C" * 1024)
        assert MultipartFileS3(str(f)).get_part(2) == b"B" * 1024

    def test_final_part_is_truncated_to_remainder(self, tiny_parts, tmp_path):
        # 1024 + 1024 + 500 bytes -> 3 parts, last is 500 bytes
        f = _write(tmp_path, "remainder.bin", b"A" * 1024 + b"B" * 1024 + b"C" * 500)
        mp = MultipartFileS3(str(f))
        assert mp.get_part_numbers() == 3
        assert mp.get_part(3) == b"C" * 500

    def test_round_trip_concatenation_reproduces_file(self, tiny_parts, tmp_path):
        """Acid test: write a file, split into parts, concatenate -> original bytes."""
        content = os.urandom(5 * 1024 + 137)  # 5 full parts + 1 partial
        f = _write(tmp_path, "roundtrip.bin", content)
        mp = MultipartFileS3(str(f))
        reassembled = b"".join(mp.get_part(n) for n in range(1, mp.get_part_numbers() + 1))
        assert reassembled == content


class TestFileName:
    def test_extracts_basename_from_nested_path(self, tmp_path):
        nested = tmp_path / "sub" / "deep"
        nested.mkdir(parents=True)
        f = nested / "archive.zip"
        f.touch()
        assert MultipartFileS3(str(f)).file_name == "archive.zip"



from abxr.devices import DevicesService
from tests.conftest import make_response


def _service(mocker):
    svc = DevicesService("https://api.xrdm.app", "tok")
    svc.client = mocker.MagicMock()
    return svc


def test_get_all_devices_uses_v3(mocker):
    svc = _service(mocker)
    svc.client.get.return_value = make_response({"data": [{"id": "d1"}], "links": {}})

    result = svc.get_all_devices()

    assert result == [{"id": "d1"}]
    url = svc.client.get.call_args[0][0]
    assert url == "https://api.xrdm.app/api/v3/devices?per_page=20"


def test_get_device_detail(mocker):
    svc = _service(mocker)
    svc.client.get.return_value = make_response({"id": "d1"})

    result = svc.get_device_detail("d1")

    assert result == {"id": "d1"}
    svc.client.get.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1",
        headers=svc.headers,
    )


def test_launch_app(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({"status": "ok"})

    svc.launch_app("d1", "a1")

    svc.client.post.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1/launch/a1",
        headers=svc.headers,
    )


def test_reboot_device(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({}, content=b"")

    svc.reboot_device("d1")

    svc.client.post.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1/reboot",
        headers=svc.headers,
    )


def test_factory_reset(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({}, content=b"")

    svc.factory_reset_device("d1")

    svc.client.post.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1/factory-reset",
        headers=svc.headers,
    )


def test_migrate_to_org(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({"status": "ok"})

    svc.migrate_device_to_org("d1", "target-org", "target-token", "group-1")

    args, kwargs = svc.client.post.call_args
    assert args[0] == "https://api.xrdm.app/api/v3/devices/d1/migrate/target-org"
    assert kwargs["json"] == {
        "targetOrganizationToken": "target-token",
        "groupId": "group-1",
    }


def test_attach_tags(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({"tags": ["a", "b"]})

    svc.attach_tags_to_device("d1", ["a", "b"])

    args, kwargs = svc.client.post.call_args
    assert args[0] == "https://api.xrdm.app/api/v3/devices/d1/tags/attach"
    assert kwargs["json"] == {"tags": ["a", "b"]}



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from enum import Enum

from abxr.api_service import ApiService
from abxr.formats import DataOutputFormats
from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"
    CREATE = "create"
    DETAILS = "details"
    UPDATE = "update"
    DELETE = "delete"


class UsersService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_all_users(self):
        url = self._url('users') + '?per_page=20'
        return self._get_all_pages(url)

    def create_user(self, first_name, last_name, email, org_role_id):
        url = self._url('users')
        payload = {
            "firstName": first_name,
            "lastName": last_name,
            "email": email,
        }
        if org_role_id is not None:
            payload["organizationRoleId"] = org_role_id

        response = self.client.post(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def get_user_detail(self, user_id):
        url = self._url('users', user_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def update_user(self, user_id, first_name, last_name):
        url = self._url('users', user_id)
        payload = {
            "firstName": first_name,
            "lastName": last_name
        }

        response = self.client.put(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def delete_user(self, user_id):
        url = self._url('users', user_id)

        response = self.client.delete(url, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = UsersService(self.args.url, self.args.token)

    def run(self):
        if self.args.users_command == Commands.LIST.value:
            users = self.service.get_all_users()
            print_formatted(self.args.format, users)

        elif self.args.users_command == Commands.CREATE.value:
            new_user = self.service.create_user(self.args.first_name, self.args.last_name, self.args.email, self.args.org_role_id)
            print_formatted(self.args.format, new_user)

        elif self.args.users_command == Commands.DETAILS.value:
            user_detail = self.service.get_user_detail(self.args.user_id)
            print_formatted(self.args.format, user_detail)

        elif self.args.users_command == Commands.UPDATE.value:
            updated_user = self.service.update_user(self.args.user_id, self.args.first_name, self.args.last_name)
            print_formatted(self.args.format, updated_user)

        elif self.args.users_command == Commands.DELETE.value:
            self.service.delete_user(self.args.user_id)



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from enum import Enum

from abxr.api_service import ApiService
from abxr.formats import DataOutputFormats
from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"
    CREATE = "create"
    DETAIL = "detail"
    UPDATE = "update"
    DELETE = "delete"


class TagsService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_all_tags(self):
        url = self._url('tags') + '?per_page=20'
        return self._get_all_pages(url)

    def create_tag(self, tag_name):
        url = self._url('tags')
        payload = {
            "name": tag_name
        }

        response = self.client.post(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def get_tag_detail(self, tag_id):
        url = self._url('tags', tag_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def update_tag(self, tag_id, tag_name):
        url = self._url('tags', tag_id)
        payload = {
            "name": tag_name
        }

        response = self.client.put(url, headers=self.headers, json=payload)
        response.raise_for_status()

        return response.json()

    def delete_tag(self, tag_id):
        url = self._url('tags', tag_id)

        response = self.client.delete(url, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = TagsService(self.args.url, self.args.token)

    def run(self):
        if self.args.tags_command == Commands.LIST.value:
            tags = self.service.get_all_tags()
            print_formatted(self.args.format, tags)

        elif self.args.tags_command == Commands.CREATE.value:
            new_tag = self.service.create_tag(self.args.name)
            print_formatted(self.args.format, new_tag)

        elif self.args.tags_command == Commands.DETAIL.value:
            tag_detail = self.service.get_tag_detail(self.args.tag_id)
            print_formatted(self.args.format, tag_detail)

        elif self.args.tags_command == Commands.UPDATE.value:
            updated_tag = self.service.update_tag(self.args.tag_id, self.args.name)
            print_formatted(self.args.format, updated_tag)

        elif self.args.tags_command == Commands.DELETE.value:
            self.service.delete_tag(self.args.tag_id)



from abxr.files import FilesService
from tests.conftest import make_response


def _service(mocker):
    svc = FilesService("https://api.xrdm.app", "tok")
    svc.client = mocker.MagicMock()
    return svc


def test_get_all_files_uses_v3(mocker):
    svc = _service(mocker)
    svc.client.get.return_value = make_response({"data": [{"id": "f1"}], "links": {}})

    result = svc.get_all_files()

    assert result == [{"id": "f1"}]
    url = svc.client.get.call_args[0][0]
    assert url == "https://api.xrdm.app/api/v3/files?per_page=20"


def test_get_file_detail(mocker):
    svc = _service(mocker)
    svc.client.get.return_value = make_response({"id": "f1"})

    result = svc.get_file_detail("f1")

    assert result == {"id": "f1"}
    svc.client.get.assert_called_once_with(
        "https://api.xrdm.app/api/v3/files/f1",
        headers=svc.headers,
    )


def test_initiate_upload_posts_payload(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({"fileId": "f1", "uploadId": "u1", "key": "k1"})

    result = svc._initiate_upload("config.json", "/sdcard")

    assert result == {"fileId": "f1", "uploadId": "u1", "key": "k1"}
    args, kwargs = svc.client.post.call_args
    assert args[0] == "https://api.xrdm.app/api/v3/files"
    assert kwargs["json"] == {"filename": "config.json", "path": "/sdcard"}


def test_initiate_upload_with_bundle_id(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({"fileId": "f1", "uploadId": "u1", "key": "k1"})

    svc._initiate_upload("config.json", "/sdcard", app_bundle_id="b1")

    assert svc.client.post.call_args.kwargs["json"]["appBundleId"] == "b1"


def test_assign_file_to_device(mocker):
    svc = _service(mocker)
    svc.client.post.return_value = make_response({}, content=b"")

    svc.assign_file_to_device("f1", "d1")

    svc.client.post.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1/files",
        json={"fileId": "f1"},
        headers=svc.headers,
    )


def test_remove_file_from_device(mocker):
    svc = _service(mocker)
    svc.client.delete.return_value = make_response({}, content=b"")

    svc.remove_file_from_device("f1", "d1")

    svc.client.delete.assert_called_once_with(
        "https://api.xrdm.app/api/v3/devices/d1/files",
        json={"fileId": "f1"},
        headers=svc.headers,
    )


class TestMultipartUpload:
    """End-to-end multipart upload: initiate -> presign -> PUT parts to S3 -> complete."""

    def test_uploads_single_part_file(self, mocker, tmp_path):
        # File smaller than MIN_PART_SIZE (5 MiB) -> exactly one part.
        f = tmp_path / "config.json"
        f.write_bytes(b'{"hello": "world"}')

        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"fileId": "file-1", "uploadId": "upload-1", "key": "key-1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3.example/part-1"}]),
            make_response({"id": "file-1", "status": "available"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"etag-1"'})

        result = svc.upload_file(str(f), "/sdcard", silent=True)

        assert result == {"id": "file-1", "status": "available"}

        initiate_call, presign_call, complete_call = svc.client.post.call_args_list
        assert initiate_call.args[0] == "https://api.xrdm.app/api/v3/files"
        assert initiate_call.kwargs["json"] == {"filename": "config.json", "path": "/sdcard"}

        assert presign_call.args[0] == "https://api.xrdm.app/api/v3/files/file-1/pre-sign"
        assert presign_call.kwargs["json"] == {
            "key": "key-1", "uploadId": "upload-1", "partNumbers": [1],
        }

        svc.client.put.assert_called_once()
        put_call = svc.client.put.call_args
        assert put_call.args[0] == "https://s3.example/part-1"
        assert put_call.kwargs["data"] == b'{"hello": "world"}'

        assert complete_call.args[0] == "https://api.xrdm.app/api/v3/files/file-1/complete"
        assert complete_call.kwargs["json"] == {
            "key": "key-1",
            "uploadId": "upload-1",
            "parts": [{"partNumber": 1, "eTag": '"etag-1"'}],
            "conflictStrategy": "replace",
        }

    def test_batches_presigned_url_requests(self, mocker, tmp_path):
        """With >4 parts, presigning is batched into chunks of MAX_PARTS_PER_REQUEST."""
        # Create a file large enough for 5 parts (5 MiB minimum each = ~25 MiB to ensure 5 parts).
        # Cheaper: monkey-patch MultipartFileS3 to report 5 small parts.
        from abxr import files as files_module

        fake = mocker.MagicMock()
        fake.file_name = "big.bin"
        fake.get_size.return_value = 5
        fake.get_part_numbers.return_value = 5
        fake.get_part.return_value = b"x"
        mocker.patch.object(files_module, "MultipartFileS3", return_value=fake)

        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"fileId": "file-1", "uploadId": "u1", "key": "k1"}),
            # First presign batch (parts 1-4)
            make_response([
                {"partNumber": n, "presignedUrl": f"https://s3/{n}"} for n in [1, 2, 3, 4]
            ]),
            # Second presign batch (part 5)
            make_response([{"partNumber": 5, "presignedUrl": "https://s3/5"}]),
            # Complete
            make_response({"id": "file-1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e"'})

        svc.upload_file("ignored", "/sdcard", silent=True)

        # 1 initiate + 2 presign + 1 complete = 4 POSTs
        assert svc.client.post.call_count == 4
        # 5 part PUTs to S3
        assert svc.client.put.call_count == 5

        # All 5 parts reported in complete payload
        complete_payload = svc.client.post.call_args_list[-1].kwargs["json"]
        assert [p["partNumber"] for p in complete_payload["parts"]] == [1, 2, 3, 4, 5]



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from tqdm import tqdm

from enum import Enum

from abxr.api_service import ApiService
from abxr.formats import DataOutputFormats

from abxr.output import print_formatted

class Commands(Enum):
    LIST = "list"
    DETAILS = "details"
    LAUNCH_APP = "launch"
    REBOOT = "reboot"
    SHUTDOWN = "shutdown"
    FACTORY_RESET = "factory_reset"

    RELEASE_CHANNELS_LIST = "release_channels"
    RELEASE_CHANNEL_ADD = "add_release_channel"
    RELEASE_CHANNEL_REMOVE = "remove_release_channel"
    FILES_LIST = "list_files"
    FILES_ADD = "add_file"
    FILES_REMOVE = "remove_file"
    VIDEOS_LIST = "list_videos"
    VIDEOS_ADD = "add_video"
    VIDEOS_REMOVE = "remove_video"

    MIGRATE_TO_ORG = "migrate_to_org"

    ATTACH_TAGS = "attach_tags"
    DETACH_TAGS = "detach_tags"

class DevicesService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_all_devices(self):
        url = self._url('devices') + '?per_page=20'
        return self._get_all_pages(url)

    def get_device_detail(self, device_id):
        url = self._url('devices', device_id)

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def launch_app(self, device_id, app_id):
        url = self._url('devices', device_id, 'launch', app_id)

        response = self.client.post(url, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def reboot_device(self, device_id):
        url = self._url('devices', device_id, 'reboot')

        response = self.client.post(url, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def shutdown_device(self, device_id):
        url = self._url('devices', device_id, 'shutdown')

        response = self.client.post(url, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def factory_reset_device(self, device_id):
        url = self._url('devices', device_id, 'factory-reset')

        response = self.client.post(url, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def list_release_channels_for_device(self, device_id):
        url = self._url('devices', device_id, 'release-channels') + '?per_page=20'
        return self._get_all_pages(url)

    def add_release_channel_to_device(self, device_id, release_channel_id):
        url = self._url('devices', device_id, 'release-channels')

        data = {
            'releaseChannelId': release_channel_id
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def remove_release_channel_from_device(self, device_id, release_channel_id):
        url = self._url('devices', device_id, 'release-channels')

        data = {
            'releaseChannelId': release_channel_id
        }

        response = self.client.delete(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def list_files_for_device(self, device_id):
        url = self._url('devices', device_id, 'files') + '?per_page=20'
        return self._get_all_pages(url)

    def add_file_to_device(self, device_id, file_id):
        url = self._url('devices', device_id, 'files')

        data = {
            'fileId': file_id
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def remove_file_from_device(self, device_id, file_id):
        url = self._url('devices', device_id, 'files')

        data = {
            'fileId': file_id
        }

        response = self.client.delete(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def list_videos_for_device(self, device_id):
        url = self._url('devices', device_id, 'videos') + '?per_page=20'
        return self._get_all_pages(url)

    def add_video_to_device(self, device_id, video_id):
        url = self._url('devices', device_id, 'videos')

        data = {
            'videoId': video_id
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def remove_video_from_device(self, device_id, video_id):
        url = self._url('devices', device_id, 'videos')

        data = {
            'videoId': video_id
        }

        response = self.client.delete(url, json=data, headers=self.headers)
        response.raise_for_status()

        return self._parse_response(response)

    def migrate_device_to_org(self, device_id, new_organization_slug, new_organization_token, new_organization_group_id):
        url = self._url('devices', device_id, 'migrate', new_organization_slug)

        data = {
            'targetOrganizationToken': new_organization_token,
            'groupId': new_organization_group_id
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def attach_tags_to_device(self, device_id, tags):
        url = self._url('devices', device_id, 'tags', 'attach')

        data = {
            'tags': tags
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

    def detach_tags_from_device(self, device_id, tags):
        url = self._url('devices', device_id, 'tags', 'detach')

        data = {
            'tags': tags
        }

        response = self.client.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()



class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = DevicesService(self.args.url, self.args.token)

    def run(self):
        if self.args.devices_command == Commands.LIST.value:
            devices_list = self.service.get_all_devices()
            print_formatted(self.args.format, devices_list)

        elif self.args.devices_command == Commands.DETAILS.value:
            device_detail = self.service.get_device_detail(self.args.device_id)
            print_formatted(self.args.format, device_detail)

        elif self.args.devices_command == Commands.LAUNCH_APP.value:
            self.service.launch_app(self.args.device_id, self.args.app_id)

        elif self.args.devices_command == Commands.REBOOT.value:
            self.service.reboot_device(self.args.device_id)

        elif self.args.devices_command == Commands.SHUTDOWN.value:
            self.service.shutdown_device(self.args.device_id)

        elif self.args.devices_command == Commands.FACTORY_RESET.value:
            self.service.factory_reset_device(self.args.device_id)

        elif self.args.devices_command == Commands.RELEASE_CHANNELS_LIST:
            device_release_channels_list = self.service.list_release_channels_for_device(self.args.device_id)
            print_formatted(self.args.format, device_release_channels_list)

        elif self.args.devices_command == Commands.RELEASE_CHANNEL_ADD:
            self.service.add_release_channel_to_device(self.args.device_id, self.args.release_channel_id)

        elif self.args.devices_command == Commands.RELEASE_CHANNEL_REMOVE:
            self.service.remove_release_channel_from_device(self.args.device_id, self.args.release_channel_id)

        elif self.args.devices_command == Commands.FILES_LIST:
            device_files_list = self.service.list_files_for_device(self.args.device_id)
            print_formatted(self.args.format, device_files_list)

        elif self.args.devices_command == Commands.FILES_ADD:
            self.service.add_file_to_device(self.args.device_id, self.args.file_id)

        elif self.args.devices_command == Commands.FILES_REMOVE:
            self.service.remove_file_from_device(self.args.device_id, self.args.file_id)

        elif self.args.devices_command == Commands.VIDEOS_LIST:
            device_videos_list = self.service.list_videos_for_device(self.args.device_id)
            print_formatted(self.args.format, device_videos_list)

        elif self.args.devices_command == Commands.VIDEOS_ADD:
            self.service.add_video_to_device(self.args.device_id, self.args.video_id)

        elif self.args.devices_command == Commands.VIDEOS_REMOVE:
            self.service.remove_video_from_device(self.args.device_id, self.args.video_id)

        elif self.args.devices_command == Commands.MIGRATE_TO_ORG:
            self.service.migrate_device_to_org(self.args.device_id, self.args.new_organization_slug, self.args.new_organization_token, self.args.new_organization_group_id)



import json

import yaml

from abxr.output import print_formatted


def test_json_format(capsys):
    print_formatted("json", {"foo": "bar"})
    captured = capsys.readouterr()
    assert json.loads(captured.out.strip()) == {"foo": "bar"}


def test_yaml_format(capsys):
    print_formatted("yaml", {"foo": "bar"})
    captured = capsys.readouterr()
    assert yaml.safe_load(captured.out) == {"foo": "bar"}


def test_invalid_format(capsys):
    print_formatted("xml", {"foo": "bar"})
    captured = capsys.readouterr()
    assert "Invalid output format" in captured.out


def test_yaml_list(capsys):
    print_formatted("yaml", [{"id": 1}, {"id": 2}])
    captured = capsys.readouterr()
    assert yaml.safe_load(captured.out) == [{"id": 1}, {"id": 2}]



#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import re
import requests

API_VERSION = 'v3'


class ApiService:
    def __init__(self, base_url, token):
        self._raw_base_url = base_url

        if re.search(r'/api/v2(/|$)', base_url):
            raise ValueError(
                f"ABXR_API_URL points at v2 ('{base_url}'), which is no longer supported. "
                f"Use the base origin (e.g. https://api.xrdm.app) or /api/v3."
            )

        # Strip any /api suffix and what follows — we only need the base origin.
        # Handles: /api/v3, /api/internal, /api, or bare origin.
        self._base_origin = re.sub(r'/api(?:/\S+)?/?$', '', base_url.rstrip('/'))

        self.headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        if ".local" in self._raw_base_url:
            requests.packages.urllib3.disable_warnings(
                requests.packages.urllib3.exceptions.InsecureRequestWarning
            )

            old_request_method = requests.Session.request
            def new_request_method(self, *args, **kwargs):
                kwargs['verify'] = False
                return old_request_method(self, *args, **kwargs)

            requests.Session.request = new_request_method

        self.client = requests

    @property
    def base_url(self):
        return self._raw_base_url

    def _url(self, *segments):
        """Build a v3 API URL for a resource.

        Example: self._url('apps', 'uuid', 'versions')
                 -> 'https://api.xrdm.app/api/v3/apps/uuid/versions'
        """
        path = '/'.join(str(s).strip('/') for s in segments)
        return f'{self._base_origin}/api/{API_VERSION}/{path}'

    def _get_all_pages(self, url):
        """Fetch all pages of a paginated API resource and return flat list.

        Follows links.next until exhausted.
        """
        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()
        json_data = response.json()
        data = json_data.get('data', [])

        while json_data.get('links', {}).get('next'):
            response = self.client.get(json_data['links']['next'], headers=self.headers)
            response.raise_for_status()
            json_data = response.json()
            data += json_data.get('data', [])

        return data

    def _parse_response(self, response):
        """Parse JSON response, returning None for empty bodies (204 No Content)."""
        return response.json() if response.content else None



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from enum import Enum

from abxr.api_service import ApiService
from abxr.output import print_formatted

class Commands(Enum):
    INFO = "info"

class OrgService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_org_info(self):
        url = self._url('current-organization')

        response = self.client.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()


class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = OrgService(self.args.url, self.args.token)

    def run(self):
        if self.args.org_command == Commands.INFO.value:
            org_info = self.service.get_org_info()
            print_formatted(self.args.format, org_info)



"""Tests for SystemAppsService.

CRITICAL: SystemAppsService uses the non-versioned /api/internal/... endpoints
with org-scoped tokens for the system-app organization. After the v2 cleanup,
this code path must continue to work unchanged — these tests are the guard.
"""

import pytest

from abxr.system_apps import SystemAppsService
from tests.conftest import make_response


def _service(mocker, base_url="https://api.xrdm.app"):
    svc = SystemAppsService(base_url, "tok")
    svc.client = mocker.MagicMock()
    return svc


class TestConstructorAcceptsInternalUrls:
    def test_bare_origin(self):
        svc = SystemAppsService("https://api.xrdm.app", "tok")
        assert svc._internal_base == "https://api.xrdm.app/api/internal"

    def test_strips_api_internal_suffix(self):
        svc = SystemAppsService("https://api.xrdm.app/api/internal", "tok")
        assert svc._internal_base == "https://api.xrdm.app/api/internal"

    def test_strips_api_v3_suffix(self):
        svc = SystemAppsService("https://api.xrdm.app/api/v3", "tok")
        assert svc._internal_base == "https://api.xrdm.app/api/internal"


class TestUrlOverride:
    def test_builds_internal_path_not_v3(self, mocker):
        svc = _service(mocker)
        url = svc._url("apps", "client", "versions")
        assert url == "https://api.xrdm.app/api/internal/apps/client/versions"
        assert "/api/v3/" not in url

    def test_join_strips_segment_slashes(self, mocker):
        svc = _service(mocker)
        url = svc._url("/apps/", "client", "versions/")
        assert url == "https://api.xrdm.app/api/internal/apps/client/versions"


class TestVersionsList:
    def test_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [{"id": "v1"}], "links": {}})

        result = svc.get_all_app_versions_by_type("client")

        assert result == [{"id": "v1"}]
        url = svc.client.get.call_args[0][0]
        assert url == "https://api.xrdm.app/api/internal/apps/client/versions?per_page=20"


class TestReleaseChannels:
    def test_list_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [], "links": {}})

        svc.get_all_release_channels_for_app("client")

        url = svc.client.get.call_args[0][0]
        assert url == "https://api.xrdm.app/api/internal/apps/client/release-channels?per_page=20"

    def test_detail_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"id": "rc1"})

        svc.get_release_channel_detail("client", "rc1")

        svc.client.get.assert_called_once_with(
            "https://api.xrdm.app/api/internal/apps/client/release-channels/rc1",
            headers=svc.headers,
        )


class TestAppCompatibilities:
    def test_list_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [], "links": {}})

        svc.get_all_app_compatibilities_for_app("client")

        url = svc.client.get.call_args[0][0]
        assert url == "https://api.xrdm.app/api/internal/apps/client/app-compatibilities?per_page=20"

    def test_detail_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"id": "c1"})

        svc.get_app_compatibility_detail("client", "c1")

        svc.client.get.assert_called_once_with(
            "https://api.xrdm.app/api/internal/apps/client/app-compatibilities/c1",
            headers=svc.headers,
        )


class TestUploadFlow:
    def test_initiate_upload_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.post.return_value = make_response({
            "versionId": "v1", "uploadId": "u1", "key": "k1",
        })

        svc._initiate_upload("client", "client.apk", "compat-1", "1.0.0", 1)

        args, kwargs = svc.client.post.call_args
        assert args[0] == "https://api.xrdm.app/api/internal/apps/client/versions"
        assert kwargs["json"]["filename"] == "client.apk"
        assert kwargs["json"]["appCompatibilityId"] == "compat-1"
        assert kwargs["json"]["versionName"] == "1.0.0"
        assert kwargs["json"]["versionCode"] == 1

    def test_initiate_upload_omits_optional_fields(self, mocker):
        svc = _service(mocker)
        svc.client.post.return_value = make_response({
            "versionId": "v1", "uploadId": "u1", "key": "k1",
        })

        svc._initiate_upload("client", "client.apk", "compat-1")

        payload = svc.client.post.call_args.kwargs["json"]
        assert "versionName" not in payload
        assert "versionCode" not in payload

    def test_presigned_url_hits_internal_endpoint(self, mocker):
        svc = _service(mocker)
        svc.client.post.return_value = make_response([{"partNumber": 1, "presignedUrl": "https://s3..."}])

        svc._presigned_url("client", "v1", "u1", "k1", [1])

        args, kwargs = svc.client.post.call_args
        assert args[0] == "https://api.xrdm.app/api/internal/apps/client/versions/v1/pre-sign"
        assert kwargs["json"] == {"key": "k1", "uploadId": "u1", "partNumbers": [1]}


class TestFullSystemAppUpload:
    """End-to-end upload flow for system apps (e.g. client.apk to /api/internal/...)."""

    def _setup_fake_file(self, mocker):
        from abxr import system_apps as system_apps_module
        fake = mocker.MagicMock()
        fake.file_name = "client.apk"
        fake.get_size.return_value = 4
        fake.get_part_numbers.return_value = 1
        fake.get_part.return_value = b"apk!"
        mocker.patch.object(system_apps_module, "MultipartFileS3", return_value=fake)

    def test_resolves_compatibility_then_uploads(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        # get_all_app_compatibilities_for_app -> internal GET
        svc.client.get.return_value = make_response({
            "data": [{"id": "compat-1", "name": "Pico 4 Enterprise"}],
            "links": {},
        })
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            # Internal API returns UPPERCASE status enum names, not v3 lowercase
            make_response({"id": "v1", "status": "AVAILABLE"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})

        result = svc.upload_file(
            "client", "/fake/client.apk", "Pico 4 Enterprise",
            "2.0.0", 200, "release notes", silent=True,
        )

        assert result == {"id": "v1", "status": "AVAILABLE"}

        # All POSTs hit /api/internal/...
        for call in svc.client.post.call_args_list:
            assert "/api/internal/apps/client/versions" in call.args[0]
            assert "/api/v3/" not in call.args[0]

        # Initiate carries the resolved compatibility id and version metadata
        initiate_payload = svc.client.post.call_args_list[0].kwargs["json"]
        assert initiate_payload["appCompatibilityId"] == "compat-1"
        assert initiate_payload["versionName"] == "2.0.0"
        assert initiate_payload["versionCode"] == 200

        # Complete carries version + release notes
        complete_payload = svc.client.post.call_args_list[-1].kwargs["json"]
        assert complete_payload["parts"] == [{"partNumber": 1, "eTag": '"e1"'}]

    def test_raises_when_compatibility_not_found(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.get.return_value = make_response({
            "data": [{"id": "compat-1", "name": "Other Headset"}],
            "links": {},
        })

        with pytest.raises(ValueError, match="App compatibility .* not found"):
            svc.upload_file(
                "client", "/fake/client.apk", "Pico 4 Enterprise",
                "2.0.0", 200, "notes", silent=True,
            )



import pytest

from abxr.apps import AppsService
from tests.conftest import make_response


def _service(mocker):
    svc = AppsService("https://api.xrdm.app", "tok")
    svc.client = mocker.MagicMock()
    return svc


class TestGetAllApps:
    def test_paginates(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [{"id": "a"}, {"id": "b"}], "links": {}})

        result = svc.get_all_apps()

        assert result == [{"id": "a"}, {"id": "b"}]
        called_url = svc.client.get.call_args[0][0]
        assert called_url == "https://api.xrdm.app/api/v3/apps?per_page=20"


class TestGetAppDetail:
    def test_hits_v3_path(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"id": "abc"})

        result = svc.get_app_detail("abc")

        assert result == {"id": "abc"}
        svc.client.get.assert_called_once_with(
            "https://api.xrdm.app/api/v3/apps/abc",
            headers=svc.headers,
        )


class TestVersionsBySha256:
    def test_empty_input_returns_empty(self, mocker):
        svc = _service(mocker)
        assert svc.get_versions_by_sha256("app-id", []) == []
        svc.client.get.assert_not_called()

    def test_filters_to_available_only(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": [
            {"id": "v1", "status": "available"},
            {"id": "v2", "status": "processing"},
            {"id": "v3", "status": "error"},
            {"id": "v4", "status": "available"},
        ]})

        result = svc.get_versions_by_sha256("app-id", ["hash1"])

        assert [v["id"] for v in result] == ["v1", "v4"]

    def test_builds_sha256_query_params(self, mocker):
        svc = _service(mocker)
        svc.client.get.return_value = make_response({"data": []})

        svc.get_versions_by_sha256("app-id", ["h1", "h2"])

        called_url = svc.client.get.call_args[0][0]
        assert "sha256[]=h1" in called_url
        assert "sha256[]=h2" in called_url
        assert called_url.startswith("https://api.xrdm.app/api/v3/apps/app-id/versions?")


class TestFilesBySha512:
    def test_empty_input_returns_empty(self, mocker):
        svc = _service(mocker)
        assert svc.get_files_by_sha512("app-id", []) == []
        svc.client.get.assert_not_called()


class TestAppUpload:
    """End-to-end APK upload exercising the v3 lowercase status polling."""

    def _setup_fake_file(self, mocker):
        from abxr import apps as apps_module
        fake = mocker.MagicMock()
        fake.file_name = "translate.apk"
        fake.get_size.return_value = 4
        fake.get_part_numbers.return_value = 1
        fake.get_part.return_value = b"apk!"
        mocker.patch.object(apps_module, "MultipartFileS3", return_value=fake)
        return fake

    def test_full_upload_and_status_polling_available(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})
        svc.client.get.return_value = make_response({
            "data": [{"id": "v1", "status": "available"}],
        })
        mocker.patch("abxr.apps.time.sleep")

        result = svc.upload_file(
            "app-1", "/fake/path.apk", "1.0.0", "notes",
            silent=True, wait=True, max_wait_time_sec=10,
        )

        assert result == {"id": "v1"}
        # initiate -> presign -> complete = 3 POSTs to API
        assert svc.client.post.call_count == 3
        # status polled at least once
        assert svc.client.get.called

    def test_status_polling_raises_on_error_status(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})
        svc.client.get.return_value = make_response({
            "data": [{"id": "v1", "status": "error"}],
        })
        mocker.patch("abxr.apps.time.sleep")

        with pytest.raises(Exception, match="Upload failed server processing"):
            svc.upload_file(
                "app-1", "/fake/path.apk", "1.0.0", "notes",
                silent=True, wait=True, max_wait_time_sec=10,
            )

    def test_status_polling_handles_processing_then_available(self, mocker):
        """Verify polling loops while status is something other than available/error."""
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})
        svc.client.get.side_effect = [
            make_response({"data": [{"id": "v1", "status": "processing"}]}),
            make_response({"data": [{"id": "v1", "status": "processing"}]}),
            make_response({"data": [{"id": "v1", "status": "available"}]}),
        ]
        mocker.patch("abxr.apps.time.sleep")

        result = svc.upload_file(
            "app-1", "/fake/path.apk", "1.0.0", "notes",
            silent=True, wait=True, max_wait_time_sec=10,
        )

        assert result == {"id": "v1"}
        assert svc.client.get.call_count == 3

    def test_wait_false_skips_status_polling(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1",
                           "appBundleId": "bundle-123"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})

        result = svc.upload_file(
            "app-1", "/fake/path.apk", "1.0.0", "notes",
            silent=True, wait=False,
        )

        # appBundleId from initiate response is propagated into complete_response
        assert result["appBundleId"] == "bundle-123"
        svc.client.get.assert_not_called()

    def test_complete_payload_carries_version_and_notes(self, mocker):
        self._setup_fake_file(mocker)
        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})

        svc.upload_file(
            "app-1", "/fake/path.apk", "1.0.0", "release notes",
            silent=True, wait=False,
        )

        complete_payload = svc.client.post.call_args_list[-1].kwargs["json"]
        assert complete_payload["versionName"] == "1.0.0"
        assert complete_payload["releaseNotes"] == "release notes"
        assert complete_payload["parts"] == [{"partNumber": 1, "eTag": '"e1"'}]


class TestAppBuildTypeOnInitiate:
    """For bundle uploads, AppsService.upload_file is called with app_build_type='app-bundle'."""

    def test_app_build_type_passed_through(self, mocker):
        from abxr import apps as apps_module
        fake = mocker.MagicMock()
        fake.file_name = "build.apk"
        fake.get_size.return_value = 1
        fake.get_part_numbers.return_value = 1
        fake.get_part.return_value = b"x"
        mocker.patch.object(apps_module, "MultipartFileS3", return_value=fake)

        svc = _service(mocker)
        svc.client.post.side_effect = [
            make_response({"versionId": "v1", "uploadId": "u1", "key": "k1"}),
            make_response([{"partNumber": 1, "presignedUrl": "https://s3/1"}]),
            make_response({"id": "v1"}),
        ]
        svc.client.put.return_value = make_response({}, headers={"ETag": '"e1"'})

        svc.upload_file(
            "app-1", "/fake/path.apk", "1.0.0", "notes",
            silent=True, wait=False, app_build_type="app-bundle",
        )

        initiate_payload = svc.client.post.call_args_list[0].kwargs["json"]
        assert initiate_payload["appBuildType"] == "app-bundle"
